@@ -16,14 +16,25 @@
  */
 
 #include <sys/utsname.h>
+#include <ctype.h>
+#include <dirent.h>
+#include <errno.h>
+#include <limits.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "queue.h"
 #include "librc.h"
+#include "helpers.h"
+#include "misc.h"
 
 #define GENDEP          RC_LIBEXECDIR "/sh/gendepends.sh"
 
-#define RC_DEPCONFIG    RC_SVCDIR "/depconfig"
-
 static const char *bootlevel = NULL;
 
 static char *
@@ -54,27 +65,23 @@ void
 rc_deptree_free(RC_DEPTREE *deptree)
 {
 	RC_DEPINFO *di;
-	RC_DEPINFO *di2;
+	RC_DEPINFO *di_save;
 	RC_DEPTYPE *dt;
-	RC_DEPTYPE *dt2;
+	RC_DEPTYPE *dt_save;
 
 	if (!deptree)
 		return;
 
-	di = TAILQ_FIRST(deptree);
-	while (di) {
-		di2 = TAILQ_NEXT(di, entries);
-		dt = TAILQ_FIRST(&di->depends);
-		while (dt) {
-			dt2 = TAILQ_NEXT(dt, entries);
+	TAILQ_FOREACH_SAFE(di, deptree, entries, di_save) {
+		TAILQ_FOREACH_SAFE(dt, &di->depends, entries, dt_save) {
+			TAILQ_REMOVE(&di->depends, dt, entries);
 			rc_stringlist_free(dt->services);
 			free(dt->type);
 			free(dt);
-			dt = dt2;
 		}
+		TAILQ_REMOVE(deptree, di, entries);
 		free(di->service);
 		free(di);
-		di = di2;
 	}
 
 	/* Use free() here since rc_deptree_free should not call itself */
@@ -93,6 +100,17 @@ get_depinfo(const RC_DEPTREE *deptree, const char *service)
 	return NULL;
 }
 
+static RC_DEPINFO *
+make_depinfo(RC_DEPTREE *deptree, const char *service)
+{
+	RC_DEPINFO *depinfo = xmalloc(sizeof(*depinfo));
+	TAILQ_INIT(&depinfo->depends);
+	depinfo->service = xstrdup(service);
+	TAILQ_INSERT_TAIL(deptree, depinfo, entries);
+
+	return depinfo;
+}
+
 static RC_DEPTYPE *
 get_deptype(const RC_DEPINFO *depinfo, const char *type)
 {
@@ -106,9 +124,37 @@ get_deptype(const RC_DEPINFO *depinfo, const char *type)
 	return NULL;
 }
 
+static RC_DEPTYPE *
+make_deptype(RC_DEPINFO *depinfo, const char *type)
+{
+	RC_DEPTYPE *deptype = xmalloc(sizeof(*deptype));
+	deptype->type = xstrdup(type);
+	deptype->services = rc_stringlist_new();
+	TAILQ_INSERT_TAIL(&depinfo->depends, deptype, entries);
+
+	return deptype;
+}
+
+#ifdef HAVE_MALLOC_EXTENDED_ATTRIBUTE
+__attribute__ ((malloc (rc_deptree_free, 1)))
+#endif
+static RC_DEPTREE *
+make_deptree(void) {
+	RC_DEPTREE *deptree = xmalloc(sizeof(*deptree));
+	TAILQ_INIT(deptree);
+	return deptree;
+}
+
 RC_DEPTREE *
 rc_deptree_load(void) {
-	return rc_deptree_load_file(RC_DEPTREE_CACHE);
+	char *deptree_cache;
+	RC_DEPTREE *deptree;
+
+	xasprintf(&deptree_cache, "%s/deptree", rc_svcdir());
+	deptree = rc_deptree_load_file(deptree_cache);
+	free(deptree_cache);
+
+	return deptree;
 }
 
 RC_DEPTREE *
@@ -119,7 +165,7 @@ rc_deptree_load_file(const char *deptree_file)
 	RC_DEPINFO *depinfo = NULL;
 	RC_DEPTYPE *deptype = NULL;
 	char *line = NULL;
-	size_t len = 0;
+	size_t size;
 	char *type;
 	char *p;
 	char *e;
@@ -128,10 +174,8 @@ rc_deptree_load_file(const char *deptree_file)
 	if (!(fp = fopen(deptree_file, "r")))
 		return NULL;
 
-	deptree = xmalloc(sizeof(*deptree));
-	TAILQ_INIT(deptree);
-	while ((rc_getline(&line, &len, fp)))
-	{
+	deptree = make_deptree();
+	while (xgetline(&line, &size, fp) != -1) {
 		p = line;
 		e = strsep(&p, "_");
 		if (!e || strcmp(e, "depinfo") != 0)
@@ -146,10 +190,7 @@ rc_deptree_load_file(const char *deptree_file)
 			e = get_shell_value(p);
 			if (!e || *e == '\0')
 				continue;
-			depinfo = xmalloc(sizeof(*depinfo));
-			TAILQ_INIT(&depinfo->depends);
-			depinfo->service = xstrdup(e);
-			TAILQ_INSERT_TAIL(deptree, depinfo, entries);
+			depinfo = make_depinfo(deptree, e);
 			deptype = NULL;
 			continue;
 		}
@@ -160,12 +201,8 @@ rc_deptree_load_file(const char *deptree_file)
 		e = get_shell_value(p);
 		if (!e || *e == '\0')
 			continue;
-		if (!deptype || strcmp(deptype->type, type) != 0) {
-			deptype = xmalloc(sizeof(*deptype));
-			deptype->services = rc_stringlist_new();
-			deptype->type = xstrdup(type);
-			TAILQ_INSERT_TAIL(&depinfo->depends, deptype, entries);
-		}
+		if (!deptype || strcmp(deptype->type, type) != 0)
+			deptype = make_deptype(depinfo, type);
 		rc_stringlist_add(deptype->services, e);
 	}
 	fclose(fp);
@@ -655,19 +692,19 @@ static const DEPPAIR deppairs[] = {
 
 static const char *const depdirs[] =
 {
-	RC_SVCDIR,
-	RC_SVCDIR "/starting",
-	RC_SVCDIR "/started",
-	RC_SVCDIR "/stopping",
-	RC_SVCDIR "/inactive",
-	RC_SVCDIR "/wasinactive",
-	RC_SVCDIR "/failed",
-	RC_SVCDIR "/hotplugged",
-	RC_SVCDIR "/daemons",
-	RC_SVCDIR "/options",
-	RC_SVCDIR "/exclusive",
-	RC_SVCDIR "/scheduled",
-	RC_SVCDIR "/tmp",
+	"",
+	"starting",
+	"started",
+	"stopping",
+	"inactive",
+	"wasinactive",
+	"failed",
+	"hotplugged",
+	"daemons",
+	"options",
+	"exclusive",
+	"scheduled",
+	"tmp",
 	NULL
 };
 
@@ -680,16 +717,23 @@ rc_deptree_update_needed(time_t *newest, char *file)
 	int i;
 	struct stat buf;
 	time_t mtime;
+	char *path;
+	char *deptree_cache, *depconfig;
+	const char *service_dir = rc_svcdir();
 
 	/* Create base directories if needed */
-	for (i = 0; depdirs[i]; i++)
-		if (mkdir(depdirs[i], 0755) != 0 && errno != EEXIST)
+	for (i = 0; depdirs[i]; i++) {
+		xasprintf(&path, "%s/%s", service_dir, depdirs[i]);
+		if (mkdir(path, 0755) != 0 && errno != EEXIST)
 			fprintf(stderr, "mkdir `%s': %s\n", depdirs[i], strerror(errno));
+		free(path);
+	}
 
 	/* Quick test to see if anything we use has changed and we have
 	 * data in our deptree. */
 
-	if (stat(RC_DEPTREE_CACHE, &buf) == 0) {
+	xasprintf(&deptree_cache, "%s/deptree", service_dir);
+	if (stat(deptree_cache, &buf) == 0) {
 		mtime = buf.st_mtime;
 	} else {
 		/* No previous cache found.
@@ -699,30 +743,30 @@ rc_deptree_update_needed(time_t *newest, char *file)
 		newer = true;
 		mtime = time(NULL);
 	}
+	free(deptree_cache);
 
-	newer |= !deep_mtime_check(RC_INITDIR,true,&mtime,file);
-	newer |= !deep_mtime_check(RC_CONFDIR,true,&mtime,file);
-#ifdef RC_PKG_INITDIR
-    newer |= !deep_mtime_check(RC_PKG_INITDIR,true,&mtime,file);
-#endif
-#ifdef RC_PKG_CONFDIR
-    newer |= !deep_mtime_check(RC_PKG_CONFDIR,true,&mtime,file);
-#endif
-#ifdef RC_LOCAL_INITDIRs
-    newer |= !deep_mtime_check(RC_LOCAL_INITDIR,true,&mtime,file);
-#endif
-#ifdef RC_LOCAL_CONFDIR
-    newer |= !deep_mtime_check(RC_LOCAL_CONFDIR,true,&mtime,file);
-#endif
-    newer |= !deep_mtime_check(RC_CONF,true,&mtime,file);
+	for (const char * const *dirs = rc_scriptdirs(); *dirs; dirs++) {
+		static const char *subdirs[] = { "init.d", "conf.d", NULL };
+		for (const char **subdir = subdirs; *subdir; subdir++) {
+			xasprintf(&path, "%s/%s", *dirs, *subdir);
+			newer |= !deep_mtime_check(path, true, &mtime, file);
+			free(path);
+		}
+	}
+
+	xasprintf(&path, "%s/rc.conf", rc_sysconfdir());
+	newer |= !deep_mtime_check(path, true, &mtime, file);
+	free(path);
 
 	/* Some init scripts dependencies change depending on config files
 	 * outside of baselayout, like syslog-ng, so we check those too. */
-	config = rc_config_list(RC_DEPCONFIG);
+	xasprintf(&depconfig, "%s/depconfig", service_dir);
+	config = rc_config_list(depconfig);
 	TAILQ_FOREACH(s, config, entries) {
 		newer |= !deep_mtime_check(s->value, true, &mtime, file);
 	}
 	rc_stringlist_free(config);
+	free(depconfig);
 
 	/* Return newest file time, if requested */
 	if ((newer) && (newest != NULL)) {
@@ -732,6 +776,35 @@ rc_deptree_update_needed(time_t *newest, char *file)
 	return newer;
 }
 
+static void
+setup_environment(void) {
+	char *scriptdirs, *env;
+	size_t env_size = 0;
+	struct utsname uts;
+
+	for (const char * const *dirs = rc_scriptdirs(); *dirs; dirs++)
+		env_size += strlen(*dirs) + sizeof(' ');
+
+	env = scriptdirs = xmalloc(env_size);
+
+	for (const char * const *dirs = rc_scriptdirs(); *dirs; dirs++) {
+		int len = snprintf(scriptdirs, env_size, "%s ", *dirs);
+		scriptdirs += len;
+		env_size -= len;
+	}
+
+	setenv("RC_SCRIPTDIRS", env, 1);
+	free(env);
+
+	/* Some init scripts need RC_LIBEXECDIR to source stuff
+	   Ideally we should be setting our full env instead */
+	if (!getenv("RC_LIBEXECDIR"))
+		setenv("RC_LIBEXECDIR", RC_LIBEXECDIR, 0);
+
+	if (uname(&uts) == 0)
+		setenv("RC_UNAME", uts.sysname, 1);
+}
+
 /* This is a 7 phase operation
    Phase 1 is a shell script which loads each init script and config in turn
    and echos their dependency info to stdout
@@ -746,6 +819,7 @@ rc_deptree_update_needed(time_t *newest, char *file)
 bool
 rc_deptree_update(void)
 {
+
 	FILE *fp;
 	RC_DEPTREE *deptree, *providers;
 	RC_DEPINFO *depinfo = NULL, *depinfo_np, *di;
@@ -753,30 +827,23 @@ rc_deptree_update(void)
 	RC_STRINGLIST *config, *dupes, *types, *sorted, *visited;
 	RC_STRING *s, *s2, *s2_np, *s3, *s4;
 	char *line = NULL;
-	size_t len = 0;
-	char *depend, *depends, *service, *type, *nosys, *onosys;
-	size_t i, k, l;
+	size_t size;
+	char *depend, *depends, *service, *type;
+	char *deptree_cache, *depconfig;
+	size_t i, l;
 	bool retval = true;
 	const char *sys = rc_sys();
-	struct utsname uts;
 	int serrno;
 
-	/* Some init scripts need RC_LIBEXECDIR to source stuff
-	   Ideally we should be setting our full env instead */
-	if (!getenv("RC_LIBEXECDIR"))
-		setenv("RC_LIBEXECDIR", RC_LIBEXECDIR, 0);
-
-	if (uname(&uts) == 0)
-		setenv("RC_UNAME", uts.sysname, 1);
 	/* Phase 1 - source all init scripts and print dependencies */
+	setup_environment();
 	if (!(fp = popen(GENDEP, "r")))
 		return false;
 
-	deptree = xmalloc(sizeof(*deptree));
-	TAILQ_INIT(deptree);
 	config = rc_stringlist_new();
-	while ((rc_getline(&line, &len, fp)))
-	{
+
+	deptree = make_deptree();
+	while (xgetline(&line, &size, fp) != -1) {
 		depends = line;
 		service = strsep(&depends, " ");
 		if (!service || !*service)
@@ -786,12 +853,8 @@ rc_deptree_update(void)
 		if (!depinfo || strcmp(depinfo->service, service) != 0) {
 			deptype = NULL;
 			depinfo = get_depinfo(deptree, service);
-			if (!depinfo) {
-				depinfo = xmalloc(sizeof(*depinfo));
-				TAILQ_INIT(&depinfo->depends);
-				depinfo->service = xstrdup(service);
-				TAILQ_INSERT_TAIL(deptree, depinfo, entries);
-			}
+			if (!depinfo)
+				depinfo = make_depinfo(deptree, service);
 		}
 
 		/* We may not have any depends */
@@ -800,20 +863,16 @@ rc_deptree_update(void)
 
 		/* Get the type */
 		if (strcmp(type, "config") != 0) {
-			if (!deptype || strcmp(deptype->type, type) != 0)
+			if (!deptype || strcmp(deptype->type, type) != 0) {
 				deptype = get_deptype(depinfo, type);
-			if (!deptype) {
-				deptype = xmalloc(sizeof(*deptype));
-				deptype->type = xstrdup(type);
-				deptype->services = rc_stringlist_new();
-				TAILQ_INSERT_TAIL(&depinfo->depends, deptype, entries);
+				if (!deptype)
+					deptype = make_deptype(depinfo, type);
 			}
 		}
 
 		/* Now add each depend to our type.
 		   We do this individually so we handle multiple spaces gracefully */
-		while ((depend = strsep(&depends, " ")))
-		{
+		while ((depend = strsep(&depends, " "))) {
 			if (depend[0] == 0)
 				continue;
 
@@ -822,9 +881,8 @@ rc_deptree_update(void)
 				continue;
 			}
 
-			/* Don't provide ourself */
-			if (strcmp(type, "iprovide") == 0 &&
-			    strcmp(depend, service) == 0)
+			/* Don't depend on ourself */
+			if (strcmp(depend, service) == 0)
 				continue;
 
 			/* .sh files are not init scripts */
@@ -866,7 +924,9 @@ rc_deptree_update(void)
 	/* Phase 2 - if we're a special system, remove services that don't
 	 * work for them. This doesn't stop them from being run directly. */
 	if (sys) {
-		len = strlen(sys);
+		char *nosys, *onosys;
+		size_t len = strlen(sys);
+
 		nosys = xmalloc(len + 2);
 		nosys[0] = '-';
 		for (i = 0; i < len; i++)
@@ -880,29 +940,30 @@ rc_deptree_update(void)
 			onosys[i + 2] = (char)tolower((unsigned char)sys[i]);
 		onosys[i + 2] = '\0';
 
-		TAILQ_FOREACH_SAFE(depinfo, deptree, entries, depinfo_np)
-			if ((deptype = get_deptype(depinfo, "keyword")))
-				TAILQ_FOREACH(s, deptype->services, entries)
-					if (strcmp(s->value, nosys) == 0 ||
-					    strcmp(s->value, onosys) == 0)
-					{
-						provide = get_deptype(depinfo, "iprovide");
-						TAILQ_REMOVE(deptree, depinfo, entries);
-						TAILQ_FOREACH(di, deptree, entries) {
-							TAILQ_FOREACH_SAFE(dt, &di->depends, entries, dt_np) {
-								rc_stringlist_delete(dt->services, depinfo->service);
-								if (provide)
-									TAILQ_FOREACH(s2, provide->services, entries)
-										rc_stringlist_delete(dt->services, s2->value);
-								if (!TAILQ_FIRST(dt->services)) {
-									TAILQ_REMOVE(&di->depends, dt, entries);
-									free(dt->type);
-									free(dt->services);
-									free(dt);
-								}
-							}
+		TAILQ_FOREACH_SAFE(depinfo, deptree, entries, depinfo_np) {
+			if (!(deptype = get_deptype(depinfo, "keyword")))
+				continue;
+			TAILQ_FOREACH(s, deptype->services, entries) {
+				if (strcmp(s->value, nosys) != 0 && strcmp(s->value, onosys) != 0)
+					continue;
+				provide = get_deptype(depinfo, "iprovide");
+				TAILQ_REMOVE(deptree, depinfo, entries);
+				TAILQ_FOREACH(di, deptree, entries) {
+					TAILQ_FOREACH_SAFE(dt, &di->depends, entries, dt_np) {
+						rc_stringlist_delete(dt->services, depinfo->service);
+						if (provide)
+							TAILQ_FOREACH(s2, provide->services, entries)
+								rc_stringlist_delete(dt->services, s2->value);
+						if (!TAILQ_FIRST(dt->services)) {
+							TAILQ_REMOVE(&di->depends, dt, entries);
+							free(dt->type);
+							free(dt->services);
+							free(dt);
 						}
 					}
+				}
+			}
+		}
 		free(nosys);
 		free(onosys);
 	}
@@ -910,24 +971,20 @@ rc_deptree_update(void)
 	/* Phase 3 - add our providers to the tree */
 	providers = xmalloc(sizeof(*providers));
 	TAILQ_INIT(providers);
-	TAILQ_FOREACH(depinfo, deptree, entries)
-		if ((deptype = get_deptype(depinfo, "iprovide")))
-			TAILQ_FOREACH(s, deptype->services, entries) {
-				TAILQ_FOREACH(di, providers, entries)
-					if (strcmp(di->service, s->value) == 0)
-						break;
-				if (!di) {
-					di = xmalloc(sizeof(*di));
-					TAILQ_INIT(&di->depends);
-					di->service = xstrdup(s->value);
-					TAILQ_INSERT_TAIL(providers, di, entries);
-				}
-			}
+	TAILQ_FOREACH(depinfo, deptree, entries) {
+		if (!(deptype = get_deptype(depinfo, "iprovide")))
+			continue;
+		TAILQ_FOREACH(s, deptype->services, entries) {
+			di = get_depinfo(providers, s->value);
+			if (!di)
+				di = make_depinfo(providers, s->value);
+		}
+	}
 	TAILQ_CONCAT(deptree, providers, entries);
 	free(providers);
 
 	/* Phase 4 - backreference our depends */
-	TAILQ_FOREACH(depinfo, deptree, entries)
+	TAILQ_FOREACH(depinfo, deptree, entries) {
 		for (i = 0; deppairs[i].depend; i++) {
 			deptype = get_deptype(depinfo, deppairs[i].depend);
 			if (!deptype)
@@ -936,33 +993,23 @@ rc_deptree_update(void)
 				di = get_depinfo(deptree, s->value);
 				if (!di) {
 					if (strcmp(deptype->type, "ineed") == 0) {
-						fprintf(stderr,
-							 "Service `%s' needs non"
-							 " existent service `%s'\n",
+						fprintf(stderr, "Service '%s' needs non existent service '%s'\n",
 							 depinfo->service, s->value);
 						dt = get_deptype(depinfo, "broken");
-						if (!dt) {
-							dt = xmalloc(sizeof(*dt));
-							dt->type = xstrdup("broken");
-							dt->services = rc_stringlist_new();
-							TAILQ_INSERT_TAIL(&depinfo->depends, dt, entries);
-						}
+						if (!dt)
+							dt = make_deptype(depinfo, "broken");
 						rc_stringlist_addu(dt->services, s->value);
 					}
 					continue;
 				}
 
 				dt = get_deptype(di, deppairs[i].addto);
-				if (!dt) {
-					dt = xmalloc(sizeof(*dt));
-					dt->type = xstrdup(deppairs[i].addto);
-					dt->services = rc_stringlist_new();
-					TAILQ_INSERT_TAIL(&di->depends, dt, entries);
-				}
+				if (!dt)
+					dt = make_deptype(di, deppairs[i].addto);
 				rc_stringlist_addu(dt->services, depinfo->service);
 			}
 		}
-
+	}
 
 	/* Phase 5 - Remove broken before directives */
 	types = rc_stringlist_new();
@@ -1035,43 +1082,42 @@ rc_deptree_update(void)
 	   This works and should be entirely shell parseable provided that depend
 	   names don't have any non shell variable characters in
 	   */
-	if ((fp = fopen(RC_DEPTREE_CACHE, "w"))) {
+	xasprintf(&deptree_cache, "%s/deptree", rc_svcdir());
+	if ((fp = fopen(deptree_cache, "w"))) {
 		i = 0;
 		TAILQ_FOREACH(depinfo, deptree, entries) {
-			fprintf(fp, "depinfo_%zu_service='%s'\n",
-				i, depinfo->service);
+			fprintf(fp, "depinfo_%zu_service='%s'\n", i, depinfo->service);
 			TAILQ_FOREACH(deptype, &depinfo->depends, entries) {
-				k = 0;
+				size_t k = 0;
 				TAILQ_FOREACH(s, deptype->services, entries) {
-					fprintf(fp,
-						"depinfo_%zu_%s_%zu='%s'\n",
-						i, deptype->type, k, s->value);
-					k++;
+					fprintf(fp, "depinfo_%zu_%s_%zu='%s'\n",
+							i, deptype->type, k++, s->value);
 				}
 			}
 			i++;
 		}
 		fclose(fp);
 	} else {
-		fprintf(stderr, "fopen `%s': %s\n",
-			RC_DEPTREE_CACHE, strerror(errno));
+		fprintf(stderr, "fopen '%s': %s\n", deptree_cache, strerror(errno));
 		retval = false;
 	}
+	free(deptree_cache);
 
 	/* Save our external config files to disk */
+	xasprintf(&depconfig, "%s/depconfig", rc_svcdir());
 	if (TAILQ_FIRST(config)) {
-		if ((fp = fopen(RC_DEPCONFIG, "w"))) {
+		if ((fp = fopen(depconfig, "w"))) {
 			TAILQ_FOREACH(s, config, entries)
 				fprintf(fp, "%s\n", s->value);
 			fclose(fp);
 		} else {
-			fprintf(stderr, "fopen `%s': %s\n",
-				RC_DEPCONFIG, strerror(errno));
+			fprintf(stderr, "fopen '%s': %s\n", depconfig, strerror(errno));
 			retval = false;
 		}
 	} else {
-		unlink(RC_DEPCONFIG);
+		unlink(depconfig);
 	}
+	free(depconfig);
 
 	rc_stringlist_free(config);
 	rc_deptree_free(deptree);
