@@ -15,15 +15,30 @@
  *    except according to the terms contained in the LICENSE file.
  */
 
+#include <helpers.h>
+#include <ctype.h>
+#include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <libgen.h>
+#include <limits.h>
+#include <regex.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/param.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include "queue.h"
 #include "librc.h"
-#include <helpers.h>
+#include "misc.h"
+#include "rc.h"
 #ifdef __FreeBSD__
 #  include <sys/sysctl.h>
 #endif
 
-#define RC_RUNLEVEL	RC_SVCDIR "/softlevel"
-
 #ifndef S_IXUGO
 #  define S_IXUGO (S_IXUSR | S_IXGRP | S_IXOTH)
 #endif
@@ -62,7 +77,7 @@ ls_dir(const char *dir, int options)
 	RC_STRINGLIST *list = NULL;
 	struct stat buf;
 	size_t l;
-	char file[PATH_MAX];
+	char *file;
 	int r;
 
 	list = rc_stringlist_new();
@@ -74,9 +89,9 @@ ls_dir(const char *dir, int options)
 				/* Check that our file really exists.
 				 * This is important as a service maybe in a
 				 * runlevel, but could have been removed. */
-				snprintf(file, sizeof(file), "%s/%s",
-				    dir, d->d_name);
+				xasprintf(&file, "%s/%s", dir, d->d_name);
 				r = stat(file, &buf);
+				free(file);
 				if (r != 0)
 					continue;
 
@@ -88,10 +103,10 @@ ls_dir(const char *dir, int options)
 					continue;
 			}
 			if (options & LS_DIR) {
-				snprintf(file, sizeof(file), "%s/%s",
-				    dir, d->d_name);
-				if (stat(file, &buf) != 0 ||
-				    !S_ISDIR(buf.st_mode))
+				xasprintf(&file, "%s/%s", dir, d->d_name);
+				r = stat(file, &buf);
+				free(file);
+				if (r != 0 || !S_ISDIR(buf.st_mode))
 					continue;
 			}
 			rc_stringlist_add(list, d->d_name);
@@ -106,7 +121,7 @@ rm_dir(const char *pathname, bool top)
 {
 	DIR *dp;
 	struct dirent *d;
-	char file[PATH_MAX];
+	char *file = NULL;
 	struct stat s;
 	bool retval = true;
 
@@ -118,8 +133,7 @@ rm_dir(const char *pathname, bool top)
 		if (strcmp(d->d_name, ".") != 0 &&
 		    strcmp(d->d_name, "..") != 0)
 		{
-			snprintf(file, sizeof(file),
-			    "%s/%s", pathname, d->d_name);
+			xasprintf(&file, "%s/%s", pathname, d->d_name);
 			if (stat(file, &s) != 0) {
 				retval = false;
 				break;
@@ -136,8 +150,11 @@ rm_dir(const char *pathname, bool top)
 					break;
 				}
 			}
+			free(file);
+			file = NULL;
 		}
 	}
+	free(file);
 	closedir(dp);
 
 	if (!retval)
@@ -156,7 +173,8 @@ file_regex(const char *file, const char *regex)
 {
 	FILE *fp;
 	char *line = NULL;
-	size_t len = 0;
+	size_t size = 0;
+	ssize_t len = 0;
 	regex_t re;
 	bool retval = true;
 	int result;
@@ -173,7 +191,7 @@ file_regex(const char *file, const char *regex)
 		return false;
 	}
 
-	while ((rc_getline(&line, &len, fp))) {
+	while ((len = xgetline(&line, &size, fp)) != -1) {
 		char *str = line;
 		/* some /proc files have \0 separated content so we have to
 		   loop through the 'line' */
@@ -215,24 +233,7 @@ get_systype(void)
 }
 
 static const char *
-detect_prefix(const char *systype)
-{
-#ifdef PREFIX
-	return RC_SYS_PREFIX;
-#else
-	if (systype) {
-		if (strcmp(systype, RC_SYS_NONE) == 0)
-			return NULL;
-		if (strcmp(systype, RC_SYS_PREFIX) == 0)
-			return RC_SYS_PREFIX;
-	}
-
-	return NULL;
-#endif
-}
-
-static const char *
-detect_container(const char *systype _unused)
+detect_container(const char *systype RC_UNUSED)
 {
 #ifdef __FreeBSD__
 	if (systype) {
@@ -240,6 +241,8 @@ detect_container(const char *systype _unused)
 		       return NULL;
 		if (strcmp(systype, RC_SYS_JAIL) == 0)
 			return RC_SYS_JAIL;
+		if (strcmp(systype, RC_SYS_PODMAN) == 0)
+			return RC_SYS_PODMAN;
 	}
 
 	int jailed = 0;
@@ -248,6 +251,9 @@ detect_container(const char *systype _unused)
 	if (sysctlbyname("security.jail.jailed", &jailed, &len, NULL, 0) == 0)
 		if (jailed == 1)
 			return RC_SYS_JAIL;
+
+	if (exists("/var/run/.containerenv"))
+		return RC_SYS_PODMAN;
 #endif
 
 #ifdef __linux__
@@ -268,6 +274,8 @@ detect_container(const char *systype _unused)
 				return RC_SYS_SYSTEMD_NSPAWN;
 		if (strcmp(systype, RC_SYS_DOCKER) == 0)
 				return RC_SYS_DOCKER;
+		if (strcmp(systype, RC_SYS_PODMAN) == 0)
+			return RC_SYS_PODMAN;
 	}
 	if (file_regex("/proc/cpuinfo", "UML"))
 		return RC_SYS_UML;
@@ -285,6 +293,8 @@ detect_container(const char *systype _unused)
 		return RC_SYS_RKT;
 	else if (file_regex("/proc/1/environ", "container=systemd-nspawn"))
 		return RC_SYS_SYSTEMD_NSPAWN;
+	else if (exists("/run/.containerenv"))
+		return RC_SYS_PODMAN;
 	else if (exists("/.dockerenv"))
 		return RC_SYS_DOCKER;
 	/* old test, I'm not sure when this was valid. */
@@ -296,7 +306,7 @@ detect_container(const char *systype _unused)
 }
 
 static const char *
-detect_vm(const char *systype _unused)
+detect_vm(const char *systype RC_UNUSED)
 {
 #ifdef __NetBSD__
 	if (systype) {
@@ -339,12 +349,9 @@ rc_sys(void)
 	const char *sys;
 
 	systype = get_systype();
-	sys = detect_prefix(systype);
+	sys = detect_container(systype);
 	if (!sys) {
-		sys = detect_container(systype);
-		if (!sys) {
-			sys = detect_vm(systype);
-		}
+		sys = detect_vm(systype);
 	}
 
 	return sys;
@@ -368,7 +375,7 @@ rc_parse_service_state(RC_SERVICE state)
 static void
 get_runlevel_chain(const char *runlevel, RC_STRINGLIST *level_list, RC_STRINGLIST *ancestor_list)
 {
-	char path[PATH_MAX];
+	char *path;
 	RC_STRINGLIST *dirs;
 	RC_STRING *d, *parent;
 	const char *nextlevel;
@@ -393,7 +400,7 @@ get_runlevel_chain(const char *runlevel, RC_STRINGLIST *level_list, RC_STRINGLIS
 	 * We can now do exactly the above procedure for our chained
 	 * runlevels.
 	 */
-	snprintf(path, sizeof(path), "%s/%s", RC_RUNLEVELDIR, runlevel);
+	xasprintf(&path, "%s/%s", rc_runleveldir(), runlevel);
 	dirs = ls_dir(path, LS_DIR);
 	TAILQ_FOREACH(d, dirs, entries) {
 		nextlevel = d->value;
@@ -416,33 +423,47 @@ get_runlevel_chain(const char *runlevel, RC_STRINGLIST *level_list, RC_STRINGLIS
 		rc_stringlist_delete(ancestor_list, nextlevel);
 	}
 	rc_stringlist_free(dirs);
+	free(path);
+}
+
+static bool
+svcdir_subpath_exists(const char *subdir)
+{
+	char *path;
+	bool found;
+	xasprintf(&path, "%s/%s", rc_svcdir(), subdir);
+	found = exists(path);
+	free(path);
+	return found;
 }
 
 bool
 rc_runlevel_starting(void)
 {
-	return exists(RC_STARTING);
+	return svcdir_subpath_exists("rc.starting");
 }
 
 bool
 rc_runlevel_stopping(void)
 {
-	return exists(RC_STOPPING);
+	return svcdir_subpath_exists("rc.stopping");
 }
 
 RC_STRINGLIST *rc_runlevel_list(void)
 {
-	return ls_dir(RC_RUNLEVELDIR, LS_DIR);
+	return ls_dir(rc_runleveldir(), LS_DIR);
 }
 
 char *
 rc_runlevel_get(void)
 {
 	FILE *fp;
+	char *softlevel;
 	char *runlevel = NULL;
 	size_t i;
 
-	if ((fp = fopen(RC_RUNLEVEL, "r"))) {
+	xasprintf(&softlevel, "%s/softlevel", rc_svcdir());
+	if ((fp = fopen(softlevel, "r"))) {
 		runlevel = xmalloc(sizeof(char) * PATH_MAX);
 		if (fgets(runlevel, PATH_MAX, fp)) {
 			i = strlen(runlevel) - 1;
@@ -452,6 +473,7 @@ rc_runlevel_get(void)
 			*runlevel = '\0';
 		fclose(fp);
 	}
+	free(softlevel);
 
 	if (!runlevel || !*runlevel) {
 		free(runlevel);
@@ -464,26 +486,35 @@ rc_runlevel_get(void)
 bool
 rc_runlevel_set(const char *runlevel)
 {
-	FILE *fp = fopen(RC_RUNLEVEL, "w");
+	char *softlevel;
+	bool ret = false;
+	FILE *fp;
 
-	if (!fp)
-		return false;
-	fprintf(fp, "%s", runlevel);
-	fclose(fp);
-	return true;
+	xasprintf(&softlevel, "%s/softlevel", rc_svcdir());
+	if ((fp = fopen(softlevel, "w"))) {
+		fprintf(fp, "%s", runlevel);
+		fclose(fp);
+		ret = true;
+	}
+
+	free(softlevel);
+	return ret;
 }
 
 bool
 rc_runlevel_exists(const char *runlevel)
 {
-	char path[PATH_MAX];
+	char *path;
 	struct stat buf;
+	int r;
 
 	if (!runlevel || strcmp(runlevel, "") == 0 || strcmp(runlevel, ".") == 0 ||
 		strcmp(runlevel, "..") == 0)
 		return false;
-	snprintf(path, sizeof(path), "%s/%s", RC_RUNLEVELDIR, runlevel);
-	if (stat(path, &buf) == 0 && S_ISDIR(buf.st_mode))
+	xasprintf(&path, "%s/%s", rc_runleveldir(), runlevel);
+	r = stat(path, &buf);
+	free(path);
+	if (r == 0 && S_ISDIR(buf.st_mode))
 		return true;
 	return false;
 }
@@ -491,22 +522,29 @@ rc_runlevel_exists(const char *runlevel)
 bool
 rc_runlevel_stack(const char *dst, const char *src)
 {
-	char d[PATH_MAX], s[PATH_MAX];
+	char *d, *s;
+	int r;
 
 	if (!rc_runlevel_exists(dst) || !rc_runlevel_exists(src))
 		return false;
-	snprintf(s, sizeof(s), "../%s", src);
-	snprintf(d, sizeof(s), "%s/%s/%s", RC_RUNLEVELDIR, dst, src);
-	return (symlink(s, d) == 0 ? true : false);
+	xasprintf(&s, "../%s", src);
+	xasprintf(&d, "%s/%s/%s", rc_runleveldir(), dst, src);
+	r = symlink(s, d);
+	free(d);
+	free(s);
+	return (r == 0);
 }
 
 bool
 rc_runlevel_unstack(const char *dst, const char *src)
 {
-	char path[PATH_MAX];
+	char *path;
+	int r;
 
-	snprintf(path, sizeof(path), "%s/%s/%s", RC_RUNLEVELDIR, dst, src);
-	return (unlink(path) == 0 ? true : false);
+	xasprintf(&path, "%s/%s/%s", rc_runleveldir(), dst, src);
+	r = unlink(path);
+	free(path);
+	return (r == 0);
 }
 
 RC_STRINGLIST *
@@ -521,13 +559,71 @@ rc_runlevel_stacks(const char *runlevel)
 	return stack;
 }
 
+static const char * const scriptdirs[] = {
+#ifdef RC_LOCAL_PREFIX
+	RC_LOCAL_PREFIX "/etc",
+#endif
+	RC_SYSCONFDIR,
+#ifdef RC_PKG_PREFIX
+	RC_PKG_PREFIX "/etc",
+#endif
+	NULL
+};
+
+const char * const *
+rc_scriptdirs(void)
+{
+	return scriptdirs;
+}
+
+const char *
+rc_sysconfdir(void)
+{
+	return RC_SYSCONFDIR;
+}
+
+const char *
+rc_runleveldir(void)
+{
+	return RC_RUNLEVELDIR;
+}
+
+const char *
+rc_svcdir(void)
+{
+	return RC_SVCDIR;
+}
+
+static ssize_t
+safe_readlink(const char *path, char **buf, size_t bufsiz)
+{
+	char *buffer;
+	ssize_t r;
+
+	for (;; bufsiz += PATH_MAX) {
+		buffer = xmalloc(bufsiz);
+		r = readlink(path, buffer, bufsiz);
+		if (r < 0) {
+			free(buffer);
+			return -errno;
+		}
+		if ((size_t)r < bufsiz) {
+			buffer[r] = '\0';
+			*buf = buffer;
+			return r;
+		}
+		free(buffer);
+		if (bufsiz >= SSIZE_MAX - PATH_MAX)
+			return -ENOMEM;
+	}
+}
+
 /* Resolve a service name to its full path */
 char *
 rc_service_resolve(const char *service)
 {
-	char buffer[PATH_MAX];
-	char file[PATH_MAX];
-	int r;
+	char *buffer;
+	char *file = NULL;
 	struct stat buf;
 
 	if (!service)
@@ -537,40 +633,29 @@ rc_service_resolve(const char *service)
 		return xstrdup(service);
 
 	/* First check started services */
-	snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s", "started", service);
+	xasprintf(&file, "%s/%s/%s", rc_svcdir(), "started", service);
 	if (lstat(file, &buf) || !S_ISLNK(buf.st_mode)) {
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-		    "inactive", service);
-		if (lstat(file, &buf) || !S_ISLNK(buf.st_mode))
-			*file = '\0';
+		free(file);
+		xasprintf(&file, "%s/%s/%s", rc_svcdir(), "inactive", service);
+		if (lstat(file, &buf) || !S_ISLNK(buf.st_mode)) {
+			free(file);
+			file = NULL;
+		}
 	}
 
-	if (*file) {
-		memset(buffer, 0, sizeof(buffer));
-		r = readlink(file, buffer, sizeof(buffer)-1);
-		if (r > 0)
-			return xstrdup(buffer);
+	if (file != NULL && safe_readlink(file, &buffer, buf.st_size + 1) >= 0) {
+		free(file);
+		return buffer;
 	}
 
-#ifdef RC_LOCAL_INITDIR
-	/* Nope, so lets see if the user has written it */
-	snprintf(file, sizeof(file), RC_LOCAL_INITDIR "/%s", service);
-	if (stat(file, &buf) == 0)
-		return xstrdup(file);
-#endif
-
-	/* System scripts take precedence over 3rd party ones */
-	snprintf(file, sizeof(file), RC_INITDIR "/%s", service);
-	if (stat(file, &buf) == 0)
-		return xstrdup(file);
-
-#ifdef RC_PKG_INITDIR
-	/* Check RC_PKG_INITDIR */
-	snprintf(file, sizeof(file), RC_PKG_INITDIR "/%s", service);
-	if (stat(file, &buf) == 0)
-		return xstrdup(file);
-#endif
+	for (const char * const *dirs = rc_scriptdirs(); *dirs; dirs++) {
+		free(file);
+		xasprintf(&file, "%s/init.d/%s", *dirs, service);
+		if (stat(file, &buf) == 0)
+			return file;
+	}
 
+	free(file);
 	return NULL;
 }
 
@@ -636,19 +721,22 @@ rc_service_extra_commands(const char *service)
 	snprintf(cmd, l, OPTSTR, svc);
 	free(svc);
 
-	if ((fp = popen(cmd, "r"))) {
-		rc_getline(&buffer, &len, fp);
+	if (!(fp = popen(cmd, "r"))) {
+		free(cmd);
+		return NULL;
+	}
+
+	if (xgetline(&buffer, &len, fp) != -1) {
 		p = buffer;
 		commands = rc_stringlist_new();
 
 		while ((token = strsep(&p, " ")))
 			if (token[0] != '\0')
 				rc_stringlist_add(commands, token);
-
-		pclose(fp);
-		free(buffer);
 	}
 
+	pclose(fp);
+	free(buffer);
 	free(cmd);
 	return commands;
 }
@@ -660,7 +748,7 @@ rc_service_description(const char *service, const char *option)
 	char *svc;
 	char *cmd;
 	char *desc = NULL;
-	size_t len = 0;
+	size_t size = 0;
 	FILE *fp;
 	size_t l;
 
@@ -675,7 +763,10 @@ rc_service_description(const char *service, const char *option)
 	snprintf(cmd, l, DESCSTR, svc, *option ? "_" : "", option);
 	free(svc);
 	if ((fp = popen(cmd, "r"))) {
-		rc_getline(&desc, &len, fp);
+		if (xgetline(&desc, &size, fp) == -1) {
+			free(desc);
+			desc = NULL;
+		}
 		pclose(fp);
 	}
 	free(cmd);
@@ -685,24 +776,26 @@ rc_service_description(const char *service, const char *option)
 bool
 rc_service_in_runlevel(const char *service, const char *runlevel)
 {
-	char file[PATH_MAX];
+	char *file;
+	bool r;
 
-	snprintf(file, sizeof(file), RC_RUNLEVELDIR "/%s/%s",
-	    runlevel, basename_c(service));
-	return exists(file);
+	xasprintf(&file, "%s/%s/%s", rc_runleveldir(), runlevel, basename_c(service));
+	r = exists(file);
+	free(file);
+	return r;
 }
 
 bool
 rc_service_mark(const char *service, const RC_SERVICE state)
 {
-	char file[PATH_MAX];
+	char *file, *was;
 	int i = 0;
 	int skip_state = -1;
 	const char *base;
 	char *init = rc_service_resolve(service);
+	const char *svcdir = rc_svcdir();
 	bool skip_wasinactive = false;
 	int s;
-	char was[PATH_MAX];
 	RC_STRINGLIST *dirs;
 	RC_STRING *dir;
 	int serrno;
@@ -717,11 +810,11 @@ rc_service_mark(const char *service, const RC_SERVICE state)
 			return false;
 		}
 
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-		    rc_parse_service_state(state), base);
+		xasprintf(&file, "%s/%s/%s", svcdir, rc_parse_service_state(state), base);
 		if (exists(file))
 			unlink(file);
 		i = symlink(init, file);
+		free(file);
 		if (i != 0) {
 			free(init);
 			return false;
@@ -744,26 +837,30 @@ rc_service_mark(const char *service, const RC_SERVICE state)
 			s != RC_SERVICE_SCHEDULED) &&
 		    (!skip_wasinactive || s != RC_SERVICE_WASINACTIVE))
 		{
-			snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-			    rc_service_state_names[i].name, base);
+			xasprintf(&file, "%s/%s/%s", svcdir, rc_service_state_names[i].name, base);
 			if (exists(file)) {
 				if ((state == RC_SERVICE_STARTING ||
 					state == RC_SERVICE_STOPPING) &&
-				    s == RC_SERVICE_INACTIVE)
+					s == RC_SERVICE_INACTIVE)
 				{
-					snprintf(was, sizeof(was),
-					    RC_SVCDIR "/%s/%s",
-					    rc_parse_service_state(RC_SERVICE_WASINACTIVE),
-					    base);
-					if (symlink(init, was) == -1)
+					xasprintf(&was, "%s/%s/%s", svcdir,
+						rc_parse_service_state(RC_SERVICE_WASINACTIVE), base);
+					i = symlink(init, was);
+					free(was);
+					if (i == -1) {
+						free(file);
+						free(init);
 						return false;
+					}
 					skip_wasinactive = true;
 				}
 				if (unlink(file) == -1) {
+					free(file);
 					free(init);
 					return false;
 				}
 			}
+			free(file);
 		}
 	}
 
@@ -772,39 +869,41 @@ rc_service_mark(const char *service, const RC_SERVICE state)
 	    state == RC_SERVICE_STOPPED ||
 	    state == RC_SERVICE_INACTIVE)
 	{
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-		    "exclusive", base);
+		xasprintf(&file, "%s/%s/%s", svcdir, "exclusive", base);
 		unlink(file);
+		free(file);
 	}
 
 	/* Remove any options and daemons the service may have stored */
 	if (state == RC_SERVICE_STOPPED) {
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-		    "options", base);
+		xasprintf(&file, "%s/%s/%s", svcdir, "options", base);
 		rm_dir(file, true);
+		free(file);
 
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-		    "daemons", base);
+		xasprintf(&file, "%s/%s/%s", svcdir, "daemons", base);
 		rm_dir(file, true);
+		free(file);
 
 		rc_service_schedule_clear(service);
 	}
 
 	/* These are final states, so remove us from scheduled */
 	if (state == RC_SERVICE_STARTED || state == RC_SERVICE_STOPPED) {
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s", "scheduled");
+		xasprintf(&file, "%s/%s", svcdir, "scheduled");
 		dirs = ls_dir(file, 0);
 		TAILQ_FOREACH(dir, dirs, entries) {
-			snprintf(was, sizeof(was), "%s/%s/%s",
-			    file, dir->value, base);
+			xasprintf(&was, "%s/%s/%s", file, dir->value, base);
 			unlink(was);
+			free(was);
 
 			/* Try and remove the dir; we don't care about errors */
-			snprintf(was, sizeof(was), "%s/%s", file, dir->value);
+			xasprintf(&was, "%s/%s", file, dir->value);
 			serrno = errno;
 			rmdir(was);
 			errno = serrno;
+			free(was);
 		}
+		free(file);
 		rc_stringlist_free(dirs);
 	}
 	free(init);
@@ -816,20 +915,21 @@ rc_service_state(const char *service)
 {
 	int i;
 	int state = RC_SERVICE_STOPPED;
-	char file[PATH_MAX];
+	char *file;
 	RC_STRINGLIST *dirs;
 	RC_STRING *dir;
 	const char *base = basename_c(service);
+	const char *svcdir = rc_svcdir();
 
 	for (i = 0; rc_service_state_names[i].name; i++) {
-		snprintf(file, sizeof(file), RC_SVCDIR "/%s/%s",
-		    rc_service_state_names[i].name, base);
+		xasprintf(&file, "%s/%s/%s", svcdir, rc_service_state_names[i].name, base);
 		if (exists(file)) {
 			if (rc_service_state_names[i].state <= 0x10)
 				state = rc_service_state_names[i].state;
 			else
 				state |= rc_service_state_names[i].state;
 		}
+		free(file);
 	}
 
 	if (state & RC_SERVICE_STARTED) {
@@ -837,17 +937,20 @@ rc_service_state(const char *service)
 			state |= RC_SERVICE_CRASHED;
 	}
 	if (state & RC_SERVICE_STOPPED) {
-		dirs = ls_dir(RC_SVCDIR "/scheduled", 0);
+		char *sched_dir;
+		xasprintf(&sched_dir, "%s/scheduled", svcdir);
+		dirs = ls_dir(sched_dir, 0);
 		TAILQ_FOREACH(dir, dirs, entries) {
-			snprintf(file, sizeof(file),
-			    RC_SVCDIR "/scheduled/%s/%s",
-			    dir->value, service);
-			if (exists(file)) {
+			xasprintf(&file, "%s/scheduled/%s/%s", svcdir, dir->value, service);
+			i = exists(file);
+			free(file);
+			if (i) {
 				state |= RC_SERVICE_SCHEDULED;
 				break;
 			}
 		}
 		rc_stringlist_free(dirs);
+		free(sched_dir);
 	}
 
 	return state;
@@ -858,11 +961,11 @@ rc_service_value_get(const char *service, const char *option)
 {
 	char *buffer = NULL;
 	size_t len = 0;
-	char file[PATH_MAX];
+	char *file;
 
-	snprintf(file, sizeof(file), RC_SVCDIR "/options/%s/%s",
-	    service, option);
+	xasprintf(&file, "%s/options/%s/%s", rc_svcdir(), service, option);
 	rc_getfile(file, &buffer, &len);
+	free(file);
 
 	return buffer;
 }
@@ -872,31 +975,34 @@ rc_service_value_set(const char *service, const char *option,
     const char *value)
 {
 	FILE *fp;
-	char file[PATH_MAX];
-	char *p = file;
+	char *file1, *file2;
 
-	p += snprintf(file, sizeof(file), RC_SVCDIR "/options/%s", service);
-	if (mkdir(file, 0755) != 0 && errno != EEXIST)
+	xasprintf(&file1, "%s/options/%s", rc_svcdir(), service);
+	if (mkdir(file1, 0755) != 0 && errno != EEXIST) {
+		free(file1);
 		return false;
+	}
 
-	snprintf(p, sizeof(file) - (p - file), "/%s", option);
+	xasprintf(&file2, "%s/%s", file1, option);
+	free(file1);
 	if (value) {
-		if (!(fp = fopen(file, "w")))
+		fp = fopen(file2, "w");
+		free(file2);
+		if (fp == NULL)
 			return false;
 		fprintf(fp, "%s", value);
 		fclose(fp);
 	} else {
-		unlink(file);
+		unlink(file2);
+		free(file2);
 	}
-		return true;
+	return true;
 }
 
-
 bool
 rc_service_schedule_start(const char *service, const char *service_to_start)
 {
-	char file[PATH_MAX];
-	char *p = file;
+	char *file1, *file2;
 	char *init;
 	bool retval;
 
@@ -904,15 +1010,17 @@ rc_service_schedule_start(const char *service, const char *service_to_start)
 	if (!service || !rc_service_exists(service_to_start))
 		return false;
 
-	p += snprintf(file, sizeof(file), RC_SVCDIR "/scheduled/%s",
-	    basename_c(service));
-	if (mkdir(file, 0755) != 0 && errno != EEXIST)
+	xasprintf(&file1, "%s/scheduled/%s", rc_svcdir(), basename_c(service));
+	if (mkdir(file1, 0755) != 0 && errno != EEXIST) {
+		free(file1);
 		return false;
+	}
 
 	init = rc_service_resolve(service_to_start);
-	snprintf(p, sizeof(file) - (p - file),
-	    "/%s", basename_c(service_to_start));
-	retval = (exists(file) || symlink(init, file) == 0);
+	xasprintf(&file2, "%s/%s", file1, basename_c(service_to_start));
+	free(file1);
+	retval = (exists(file2) || symlink(init, file2) == 0);
+	free(file2);
 	free(init);
 	return retval;
 }
@@ -920,11 +1028,13 @@ rc_service_schedule_start(const char *service, const char *service_to_start)
 bool
 rc_service_schedule_clear(const char *service)
 {
-	char dir[PATH_MAX];
+	char *dir;
+	bool r;
 
-	snprintf(dir, sizeof(dir), RC_SVCDIR "/scheduled/%s",
-	    basename_c(service));
-	if (!rm_dir(dir, true) && errno == ENOENT)
+	xasprintf(&dir, "%s/scheduled/%s", rc_svcdir(), basename_c(service));
+	r = rm_dir(dir, true);
+	free(dir);
+	if (!r && errno == ENOENT)
 		return true;
 	return false;
 }
@@ -932,34 +1042,30 @@ rc_service_schedule_clear(const char *service)
 RC_STRINGLIST *
 rc_services_in_runlevel(const char *runlevel)
 {
-	char dir[PATH_MAX];
+	char *dir;
 	RC_STRINGLIST *list = NULL;
 
 	if (!runlevel) {
-#ifdef RC_PKG_INITDIR
-		RC_STRINGLIST *pkg = ls_dir(RC_PKG_INITDIR, LS_INITD);
-#endif
-#ifdef RC_LOCAL_INITDIR
-		RC_STRINGLIST *local = ls_dir(RC_LOCAL_INITDIR, LS_INITD);
-#endif
+		list = rc_stringlist_new();
+		for (const char * const *dirs = rc_scriptdirs(); *dirs; dirs++) {
+			char *initd;
+			RC_STRINGLIST *svcs;
 
-		list = ls_dir(RC_INITDIR, LS_INITD);
+			xasprintf(&initd, "%s/init.d", *dirs);
+			svcs = ls_dir(initd, LS_INITD);
+			TAILQ_CONCAT(list, svcs, entries);
 
-#ifdef RC_PKG_INITDIR
-		TAILQ_CONCAT(list, pkg, entries);
-		rc_stringlist_free(pkg);
-#endif
-#ifdef RC_LOCAL_INITDIR
-		TAILQ_CONCAT(list, local, entries);
-		rc_stringlist_free(local);
-#endif
+			rc_stringlist_free(svcs);
+			free(initd);
+		}
 		return list;
 	}
 
 	/* These special levels never contain any services */
 	if (strcmp(runlevel, RC_LEVEL_SINGLE) != 0) {
-		snprintf(dir, sizeof(dir), RC_RUNLEVELDIR "/%s", runlevel);
+		xasprintf(&dir, "%s/%s", rc_runleveldir(), runlevel);
 		list = ls_dir(dir, LS_INITD);
+		free(dir);
 	}
 	if (!list)
 		list = rc_stringlist_new();
@@ -990,29 +1096,33 @@ rc_services_in_state(RC_SERVICE state)
 	RC_STRINGLIST *list;
 	RC_STRINGLIST *dirs;
 	RC_STRING *d;
-	char dir[PATH_MAX];
-	char *p = dir;
+	char *dir, *dir2;
 
-	p += snprintf(dir, sizeof(dir), RC_SVCDIR "/%s",
-	    rc_parse_service_state(state));
-
-	if (state != RC_SERVICE_SCHEDULED)
-		return ls_dir(dir, LS_INITD);
+	xasprintf(&dir, "%s/%s", rc_svcdir(), rc_parse_service_state(state));
+	if (state != RC_SERVICE_SCHEDULED) {
+		dirs = ls_dir(dir, LS_INITD);
+		free(dir);
+		return dirs;
+	}
 
 	dirs = ls_dir(dir, 0);
 	list = rc_stringlist_new();
-	if (!dirs)
+	if (!dirs) {
+		free(dir);
 		return list;
+	}
 
 	TAILQ_FOREACH(d, dirs, entries) {
-		snprintf(p, sizeof(dir) - (p - dir), "/%s", d->value);
-		services = ls_dir(dir, LS_INITD);
+		xasprintf(&dir2, "%s/%s", dir, d->value);
+		services = ls_dir(dir2, LS_INITD);
+		free(dir2);
 		if (services) {
 			TAILQ_CONCAT(list, services, entries);
 			rc_stringlist_free(services);
 		}
 	}
 	rc_stringlist_free(dirs);
+	free(dir);
 	return list;
 }
 
@@ -1021,9 +1131,9 @@ rc_service_add(const char *runlevel, const char *service)
 {
 	bool retval;
 	char *init;
-	char file[PATH_MAX];
-	char path[MAXPATHLEN] = { '\0' };
-	char binit[PATH_MAX];
+	char *file;
+	char *path;
+	char *binit = NULL;
 	char *i;
 
 	if (!rc_runlevel_exists(runlevel)) {
@@ -1037,26 +1147,31 @@ rc_service_add(const char *runlevel, const char *service)
 	}
 
 	i = init = rc_service_resolve(service);
-	snprintf(file, sizeof(file), RC_RUNLEVELDIR "/%s/%s",
-	    runlevel, basename_c(service));
 
 	/* We need to ensure that only things in /etc/init.d are added
 	 * to the boot runlevel */
 	if (strcmp(runlevel, RC_LEVEL_BOOT) == 0) {
-		if (realpath(dirname(init), path) == NULL) {
+		path = realpath(dirname(init), NULL);
+		if (path == NULL) {
 			free(init);
 			return false;
 		}
-		if (strcmp(path, RC_INITDIR) != 0) {
+		retval = (strcmp(path, RC_INITDIR) == 0);
+		free(path);
+		if (!retval) {
 			free(init);
 			errno = EPERM;
 			return false;
 		}
-		snprintf(binit, sizeof(binit), RC_INITDIR "/%s", service);
+		xasprintf(&binit, "%s/%s", RC_INITDIR, service);
 		i = binit;
 	}
 
+	xasprintf(&file, "%s/%s/%s", rc_runleveldir(), runlevel,
+		basename_c(service));
 	retval = (symlink(i, file) == 0);
+	free(file);
+	free(binit);
 	free(init);
 	return retval;
 }
@@ -1064,39 +1179,47 @@ rc_service_add(const char *runlevel, const char *service)
 bool
 rc_service_delete(const char *runlevel, const char *service)
 {
-	char file[PATH_MAX];
+	char *file;
+	int r;
 
-	snprintf(file, sizeof(file), RC_RUNLEVELDIR "/%s/%s",
-	    runlevel, basename_c(service));
-	if (unlink(file) == 0)
-		return true;
-	return false;
+	xasprintf(&file, "%s/%s/%s", rc_runleveldir(), runlevel,
+		basename_c(service));
+	r = unlink(file);
+	free(file);
+	return (r == 0);
 }
 
 RC_STRINGLIST *
 rc_services_scheduled_by(const char *service)
 {
-	RC_STRINGLIST *dirs = ls_dir(RC_SVCDIR "/scheduled", 0);
 	RC_STRINGLIST *list = rc_stringlist_new();
+	RC_STRINGLIST *dirs;
 	RC_STRING *dir;
-	char file[PATH_MAX];
+	char *sched_dir;
+	char *file;
+
+	xasprintf(&sched_dir, "%s/scheduled", rc_svcdir());
+	dirs = ls_dir(sched_dir, 0);
 
 	TAILQ_FOREACH(dir, dirs, entries) {
-		snprintf(file, sizeof(file), RC_SVCDIR "/scheduled/%s/%s",
-		    dir->value, service);
+		xasprintf(&file, "%s/%s/%s", sched_dir, dir->value, service);
 		if (exists(file))
 			rc_stringlist_add(list, file);
+		free(file);
 	}
 	rc_stringlist_free(dirs);
+	free(sched_dir);
 	return list;
 }
 
 RC_STRINGLIST *
 rc_services_scheduled(const char *service)
 {
-	char dir[PATH_MAX];
+	char *dir;
+	RC_STRINGLIST *list;
 
-	snprintf(dir, sizeof(dir), RC_SVCDIR "/scheduled/%s",
-	    basename_c(service));
-	return ls_dir(dir, LS_INITD);
+	xasprintf(&dir, "%s/scheduled/%s", rc_svcdir(), basename_c(service));
+	list = ls_dir(dir, LS_INITD);
+	free(dir);
+	return list;
 }
