@@ -6,11 +6,11 @@
 /*
  * Copyright (c) 2007-2015 The OpenRC Authors.
  * See the Authors file at the top-level directory of this distribution and
- * https://github.com/OpenRC/openrc/blob/master/AUTHORS
+ * https://github.com/OpenRC/openrc/blob/HEAD/AUTHORS
  *
  * This file is part of OpenRC. It is subject to the license terms in
  * the LICENSE file found in the top-level directory of this
- * distribution and at https://github.com/OpenRC/openrc/blob/master/LICENSE
+ * distribution and at https://github.com/OpenRC/openrc/blob/HEAD/LICENSE
  * This file may not be copied, modified, propagated, or distributed
  *    except according to the terms contained in the LICENSE file.
  */
@@ -25,7 +25,6 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/wait.h>
-
 #include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
@@ -54,6 +53,6 @@
 #endif
 
 #include "rc.h"
-#include "rc-misc.h"
+#include "misc.h"
 
 #endif
