@@ -54,8 +54,6 @@
 #include "_usage.h"
 #include "helpers.h"
 
-#define PREFIX_LOCK	RC_SVCDIR "/prefix.lock"
-
 #define WAIT_INTERVAL	20000000	/* usecs to poll the lock file */
 #define WAIT_TIMEOUT	60		/* seconds until we timeout */
 #define WARN_TIMEOUT	10		/* warn about this every N seconds */
@@ -92,7 +90,7 @@ static RC_STRINGLIST *use_services;
 static RC_STRINGLIST *want_services;
 static RC_HOOK hook_out;
 static int exclusive_fd = -1, master_tty = -1;
-static bool sighup, in_background, deps, dry_run;
+static bool sighup, skip_mark, in_background, deps, dry_run;
 static pid_t service_pid;
 static int signal_pipe[2] = { -1, -1 };
 
@@ -117,6 +115,10 @@ handle_signal(int sig)
 		sighup = true;
 		break;
 
+	case SIGUSR1:
+		skip_mark = true;
+		break;
+
 	case SIGCHLD:
 		while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
 			if (signal_pipe[1] > -1 && pid == service_pid) {
@@ -165,7 +167,7 @@ unhotplug(void)
 {
 	char *file = NULL;
 
-	xasprintf(&file, RC_SVCDIR "/hotplugged/%s", applet);
+	xasprintf(&file, "%s/hotplugged/%s", rc_svcdir(), applet);
 	if (exists(file) && unlink(file) != 0)
 		eerror("%s: unlink `%s': %s", applet, file, strerror(errno));
 	free(file);
@@ -284,6 +286,7 @@ write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 	size_t i, j;
 	const char *ec = ecolor(ECOLOR_HILITE);
 	const char *ec_normal = ecolor(ECOLOR_NORMAL);
+	char *prefix_lock;
 	ssize_t ret = 0;
 	int fd = fileno(stdout), lock_fd = -1;
 
@@ -291,7 +294,9 @@ write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 	 * Lock the prefix.
 	 * open() may fail here when running as user, as RC_SVCDIR may not be writable.
 	 */
-	lock_fd = open(PREFIX_LOCK, O_WRONLY | O_CREAT, 0664);
+	xasprintf(&prefix_lock, "%s/prefix.lock", rc_svcdir());
+	lock_fd = open(prefix_lock, O_WRONLY | O_CREAT, 0664);
+	free(prefix_lock);
 
 	if (lock_fd != -1) {
 		while (flock(lock_fd, LOCK_EX) != 0) {
@@ -334,6 +339,18 @@ write_prefix(const char *buffer, size_t bytes, bool *prefixed)
 	return ret;
 }
 
+static int
+openrc_sh_exec(const char *openrc_sh, const char *arg1, const char *arg2)
+{
+	if (arg2)
+		einfov("Executing: %s %s %s %s %s", openrc_sh, openrc_sh, service, arg1, arg2);
+	else
+		einfov("Executing: %s %s %s %s", openrc_sh, openrc_sh, service, arg1);
+	execl(openrc_sh, openrc_sh, service, arg1, arg2, (char *) NULL);
+	eerror("%s: exec '%s': %s", service, openrc_sh, strerror(errno));
+	_exit(EXIT_FAILURE);
+}
+
 static int
 svc_exec(const char *arg1, const char *arg2)
 {
@@ -385,44 +402,20 @@ svc_exec(const char *arg1, const char *arg2)
 	if (service_pid == -1)
 		eerrorx("%s: fork: %s", service, strerror(errno));
 	if (service_pid == 0) {
+		char *openrc_sh;
+		xasprintf(&openrc_sh, "%s/openrc-run.sh", rc_svcdir());
+
 		if (slave_tty >= 0) {
 			dup2(slave_tty, STDOUT_FILENO);
 			dup2(slave_tty, STDERR_FILENO);
 		}
 
-		if (exists(RC_SVCDIR "/openrc-run.sh")) {
-			if (arg2)
-				einfov("Executing: %s %s %s %s %s",
-					RC_SVCDIR "/openrc-run.sh", RC_SVCDIR "/openrc-run.sh",
-					service, arg1, arg2);
-			else
-				einfov("Executing: %s %s %s %s",
-					RC_SVCDIR "/openrc-run.sh", RC_SVCDIR "/openrc-run.sh",
-					service, arg1);
-			execl(RC_SVCDIR "/openrc-run.sh",
-			    RC_SVCDIR "/openrc-run.sh",
-			    service, arg1, arg2, (char *) NULL);
-			eerror("%s: exec `" RC_SVCDIR "/openrc-run.sh': %s",
-			    service, strerror(errno));
-			_exit(EXIT_FAILURE);
-		} else {
-			if (arg2)
-				einfov("Executing: %s %s %s %s %s",
-					RC_LIBEXECDIR "/sh/openrc-run.sh",
-					RC_LIBEXECDIR "/sh/openrc-run.sh",
-			    	service, arg1, arg2);
-			else
-				einfov("Executing: %s %s %s %s",
-					RC_LIBEXECDIR "/sh/openrc-run.sh",
-					RC_LIBEXECDIR "/sh/openrc-run.sh",
-			    	service, arg1);
-			execl(RC_LIBEXECDIR "/sh/openrc-run.sh",
-			    RC_LIBEXECDIR "/sh/openrc-run.sh",
-			    service, arg1, arg2, (char *) NULL);
-			eerror("%s: exec `" RC_LIBEXECDIR "/sh/openrc-run.sh': %s",
-			    service, strerror(errno));
-			_exit(EXIT_FAILURE);
-		}
+		if (exists(openrc_sh))
+			openrc_sh_exec(openrc_sh, arg1, arg2);
+		else
+			openrc_sh_exec(RC_LIBEXECDIR "/sh/openrc-run.sh", arg1, arg2);
+
+		/* UNREACHABLE */
 	}
 
 	buffer = xmalloc(sizeof(char) * BUFSIZ);
@@ -508,7 +501,7 @@ svc_wait(const char *svc)
 		forever = true;
 	rc_stringlist_free(keywords);
 
-	xasprintf(&file, RC_SVCDIR "/exclusive/%s", basename_c(svc));
+	xasprintf(&file, "%s/exclusive/%s", rc_svcdir(), basename_c(svc));
 
 	interval.tv_sec = 0;
 	interval.tv_nsec = WAIT_INTERVAL;
@@ -802,6 +795,7 @@ static void svc_start_real(void)
 		setenv("IN_BACKGROUND", ibsave, 1);
 	hook_out = RC_HOOK_SERVICE_START_DONE;
 	rc_plugin_run(RC_HOOK_SERVICE_START_NOW, applet);
+	skip_mark = false;
 	started = (svc_exec("start", NULL) == 0);
 	if (ibsave)
 		unsetenv("IN_BACKGROUND");
@@ -811,7 +805,8 @@ static void svc_start_real(void)
 	else if (!started)
 		eerrorx("ERROR: %s failed to start", applet);
 
-	rc_service_mark(service, RC_SERVICE_STARTED);
+	if (!skip_mark)
+		rc_service_mark(service, RC_SERVICE_STARTED);
 	exclusive_fd = svc_unlock(applet, exclusive_fd);
 	hook_out = RC_HOOK_SERVICE_START_OUT;
 	rc_plugin_run(RC_HOOK_SERVICE_START_DONE, applet);
@@ -1008,6 +1003,7 @@ svc_stop_real(void)
 		setenv("IN_BACKGROUND", ibsave, 1);
 	hook_out = RC_HOOK_SERVICE_STOP_DONE;
 	rc_plugin_run(RC_HOOK_SERVICE_STOP_NOW, applet);
+	skip_mark = false;
 	stopped = (svc_exec("stop", NULL) == 0);
 	if (ibsave)
 		unsetenv("IN_BACKGROUND");
@@ -1015,11 +1011,12 @@ svc_stop_real(void)
 	if (!stopped)
 		eerrorx("ERROR: %s failed to stop", applet);
 
-	if (in_background)
-		rc_service_mark(service, RC_SERVICE_INACTIVE);
-	else
-		rc_service_mark(service, RC_SERVICE_STOPPED);
-
+	if (!skip_mark) {
+		if (in_background)
+		    rc_service_mark(service, RC_SERVICE_INACTIVE);
+		else
+		    rc_service_mark(service, RC_SERVICE_STOPPED);
+	}
 	hook_out = RC_HOOK_SERVICE_STOP_OUT;
 	rc_plugin_run(RC_HOOK_SERVICE_STOP_DONE, applet);
 	hook_out = 0;
@@ -1275,6 +1272,7 @@ int main(int argc, char **argv)
 
 	/* Setup a signal handler */
 	signal_setup(SIGHUP, handle_signal);
+	signal_setup(SIGUSR1, handle_signal);
 	signal_setup(SIGINT, handle_signal);
 	signal_setup(SIGQUIT, handle_signal);
 	signal_setup(SIGTERM, handle_signal);
