@@ -70,8 +70,6 @@ const char *usagestring = ""					\
 #define INITSH                  RC_LIBEXECDIR "/sh/init.sh"
 #define INITEARLYSH             RC_LIBEXECDIR "/sh/init-early.sh"
 
-#define INTERACTIVE             RC_SVCDIR "/interactive"
-
 #define DEVBOOT			"/dev/.rcboot"
 
 const char *applet = NULL;
@@ -95,16 +93,19 @@ clean_failed(void)
 	DIR *dp;
 	struct dirent *d;
 	char *path;
+	char *failed_dir;
+
+	xasprintf(&failed_dir, "%s/failed", rc_svcdir());
 
 	/* Clean the failed services state dir now */
-	if ((dp = opendir(RC_SVCDIR "/failed"))) {
+	if ((dp = opendir(failed_dir))) {
 		while ((d = readdir(dp))) {
 			if (d->d_name[0] == '.' &&
 			    (d->d_name[1] == '\0' ||
 				(d->d_name[1] == '.' && d->d_name[2] == '\0')))
 				continue;
 
-			xasprintf(&path, RC_SVCDIR "/failed/%s", d->d_name);
+			xasprintf(&path, "%s/%s", failed_dir, d->d_name);
 			if (unlink(path))
 				eerror("%s: unlink `%s': %s",
 				    applet, path, strerror(errno));
@@ -112,12 +113,16 @@ clean_failed(void)
 		}
 		closedir(dp);
 	}
+
+	free(failed_dir);
 }
 
 static void
 cleanup(void)
 {
 	RC_PID *p, *tmp;
+	const char *svcdir = rc_svcdir();
+	static const char *subdirs[] = { "rc.starting", "rc.stopping" };
 
 	if (!rc_in_logger && !rc_in_plugin &&
 	    applet && (strcmp(applet, "rc") == 0 || strcmp(applet, "openrc") == 0))
@@ -133,8 +138,13 @@ cleanup(void)
 		}
 
 		/* Clean runlevel start, stop markers */
-		rmdir(RC_STARTING);
-		rmdir(RC_STOPPING);
+		for (size_t i = 0; i < ARRAY_SIZE(subdirs); i++) {
+			char *path;
+			xasprintf(&path, "%s/%s", svcdir, subdirs[i]);
+			rmdir(path);
+			free(path);
+		}
+
 		clean_failed();
 		rc_logger_close();
 	}
@@ -211,9 +221,14 @@ want_interactive(void)
 static void
 mark_interactive(void)
 {
-	FILE *fp = fopen(INTERACTIVE, "w");
-	if (fp)
+	FILE *fp;
+	char *dir;
+
+	xasprintf(&dir, "%s/interactive", rc_svcdir());
+	if ((fp = fopen(dir, "w")))
 		fclose(fp);
+
+	free(dir);
 }
 
 static void
@@ -335,11 +350,11 @@ static char *get_krunlevel(void)
 		return NULL;
 	}
 
-	if (getline(&buffer, &i, fp) != -1) {
-		i = strlen(buffer);
-		if (buffer[i - 1] == '\n')
-			buffer[i - 1] = 0;
+	if (xgetline(&buffer, &i, fp) == -1) {
+		free(buffer);
+		buffer = NULL;
 	}
+
 	fclose(fp);
 	return buffer;
 }
@@ -543,12 +558,13 @@ do_stop_services(RC_STRINGLIST *types_nw, RC_STRINGLIST *start_services,
 	pid_t pid;
 	RC_STRING *service, *svc1, *svc2;
 	RC_STRINGLIST *deporder, *tmplist, *kwords;
+	RC_STRINGLIST *types_nw_save = NULL;
 	RC_SERVICE state;
 	RC_STRINGLIST *nostop;
 	bool crashed, nstop;
 
 	if (!types_nw) {
-		types_nw = rc_stringlist_new();
+		types_nw = types_nw_save = rc_stringlist_new();
 		rc_stringlist_add(types_nw, "needsme");
 		rc_stringlist_add(types_nw, "wantsme");
 	}
@@ -638,6 +654,9 @@ stop:
 		}
 	}
 
+	if (types_nw_save)
+		rc_stringlist_free(types_nw_save);
+
 	rc_stringlist_free(nostop);
 }
 
@@ -649,9 +668,11 @@ do_start_services(const RC_STRINGLIST *start_services, bool parallel)
 	bool interactive = false;
 	RC_SERVICE state;
 	bool crashed = false;
+	char *interactive_dir;
 
+	xasprintf(&interactive_dir, "%s/interactive", rc_svcdir());
 	if (!rc_yesno(getenv("EINFO_QUIET")))
-		interactive = exists(INTERACTIVE);
+		interactive = exists(interactive_dir);
 	errno = 0;
 	crashed = rc_conf_yesno("rc_crashed_start");
 	if (errno == ENOENT)
@@ -706,14 +727,13 @@ do_start_services(const RC_STRINGLIST *start_services, bool parallel)
 
 	/* Store our interactive status for boot */
 	if (interactive &&
-	    (strcmp(runlevel, RC_LEVEL_SYSINIT) == 0 ||
-		strcmp(runlevel, getenv("RC_BOOTLEVEL")) == 0))
+			(strcmp(runlevel, RC_LEVEL_SYSINIT) == 0 ||
+			 strcmp(runlevel, getenv("RC_BOOTLEVEL")) == 0))
 		mark_interactive();
-	else {
-		if (exists(INTERACTIVE))
-			unlink(INTERACTIVE);
-	}
-
+	else
+		if (exists(interactive_dir))
+			unlink(interactive_dir);
+	free(interactive_dir);
 }
 
 #ifdef RC_DEBUG
@@ -752,6 +772,9 @@ int main(int argc, char **argv)
 	RC_STRING *service;
 	bool going_down = false;
 	int depoptions = RC_DEP_STRICT | RC_DEP_TRACE;
+	const char *svcdir = rc_svcdir();
+	char *rc_starting, *rc_stopping;
+	char *deptree_skewed;
 	char *krunlevel = NULL;
 	char *pidstr = NULL;
 	int opt;
@@ -791,8 +814,7 @@ int main(int argc, char **argv)
 
 	/* complain about old configuration settings if they exist */
 	if (exists(RC_CONF_OLD)) {
-		ewarn("%s still exists on your system and should be removed.",
-				RC_CONF_OLD);
+		ewarn("%s still exists on your system and should be removed.", RC_CONF_OLD);
 		ewarn("Please migrate to the appropriate settings in %s", RC_CONF);
 	}
 
@@ -952,17 +974,21 @@ int main(int argc, char **argv)
 	/* Load our deptree */
 	if ((main_deptree = _rc_deptree_load(0, &regen)) == NULL)
 		eerrorx("failed to load deptree");
-	if (exists(RC_DEPTREE_SKEWED))
+
+	xasprintf(&deptree_skewed, "%s/clock-skewed", svcdir);
+	if (exists(deptree_skewed))
 		ewarn("WARNING: clock skew detected!");
+	free(deptree_skewed);
 
 	/* Clean the failed services state dir */
 	clean_failed();
 
-	if (mkdir(RC_STOPPING, 0755) != 0) {
+	xasprintf(&rc_stopping, "%s/rc.stopping", svcdir);
+	if (mkdir(rc_stopping, 0755) != 0) {
 		if (errno == EACCES)
 			eerrorx("%s: superuser access required", applet);
-		eerrorx("%s: failed to create stopping dir `%s': %s",
-		    applet, RC_STOPPING, strerror(errno));
+		eerrorx("%s: failed to create stopping dir '%s': %s",
+		    applet, rc_stopping, strerror(errno));
 	}
 
 	/* Create a list of all services which we could stop (assuming
@@ -1044,7 +1070,8 @@ int main(int argc, char **argv)
 	    going_down ? newlevel : runlevel);
 	hook_out = 0;
 
-	rmdir(RC_STOPPING);
+	rmdir(rc_stopping);
+	free(rc_stopping);
 
 	/* Store the new runlevel */
 	if (newlevel) {
@@ -1061,7 +1088,10 @@ int main(int argc, char **argv)
 		rc_logger_close();
 #endif
 
-	mkdir(RC_STARTING, 0755);
+	xasprintf(&rc_starting, "%s/rc.starting", svcdir);
+	mkdir(rc_starting, 0755);
+	free(rc_starting);
+
 	rc_plugin_run(RC_HOOK_RUNLEVEL_START_IN, runlevel);
 	hook_out = RC_HOOK_RUNLEVEL_START_OUT;
 
@@ -1130,8 +1160,12 @@ int main(int argc, char **argv)
 	 * we need to delete them so that they are regenerated again in the
 	 * default runlevel as they may depend on things that are now
 	 * available */
-	if (regen && strcmp(runlevel, bootlevel) == 0)
-		unlink(RC_DEPTREE_CACHE);
+	if (regen && strcmp(runlevel, bootlevel) == 0) {
+		char *deptree_cache;
+		xasprintf(&deptree_cache, "%s/deptree", svcdir);
+		unlink(deptree_cache);
+		free(deptree_cache);
+	}
 
 	return EXIT_SUCCESS;
 }
