@@ -73,6 +73,7 @@ env_filter(void)
 	RC_STRINGLIST *env_allow;
 	RC_STRINGLIST *profile;
 	RC_STRINGLIST *env_list;
+	char *profile_path;
 	RC_STRING *env;
 	char *e;
 	size_t i = 0;
@@ -87,7 +88,10 @@ env_filter(void)
 		rc_stringlist_free(env_allow);
 		return;
 	}
-	profile = rc_config_load(RC_PROFILE_ENV);
+
+	xasprintf(&profile_path, "%s/profile.env", rc_sysconfdir());
+	profile = rc_config_load(profile_path);
+	free(profile_path);
 
 	/* Copy the env and work from this so we can manipulate it safely */
 	env_list = rc_stringlist_new();
@@ -143,7 +147,9 @@ env_config(void)
 	char *npp;
 	char *tok;
 	const char *sys = rc_sys();
+	const char *svcdir = rc_svcdir();
 	char *buffer = NULL;
+	char *tmpdir;
 	size_t size = 0;
 
 	/* Ensure our PATH is prefixed with the system locations first
@@ -173,34 +179,31 @@ env_config(void)
 		free(e);
 	}
 
+	xasprintf(&tmpdir, "%s/tmp", svcdir);
+	e = rc_runlevel_get();
+
 	setenv("RC_VERSION", VERSION, 1);
 	setenv("RC_LIBEXECDIR", RC_LIBEXECDIR, 1);
-	setenv("RC_SVCDIR", RC_SVCDIR, 1);
-	setenv("RC_TMPDIR", RC_SVCDIR "/tmp", 1);
+	setenv("RC_SVCDIR", svcdir, 1);
+	setenv("RC_TMPDIR", tmpdir, 1);
 	setenv("RC_BOOTLEVEL", RC_LEVEL_BOOT, 1);
-	e = rc_runlevel_get();
 	setenv("RC_RUNLEVEL", e, 1);
+
 	free(e);
+	free(tmpdir);
 
 	if ((fp = fopen(RC_KRUNLEVEL, "r"))) {
-		if (getline(&buffer, &size, fp) != -1) {
-			l = strlen (buffer) - 1;
-			if (buffer[l] == '\n')
-				buffer[l] = 0;
+		if (xgetline(&buffer, &size, fp) != -1)
 			setenv("RC_DEFAULTLEVEL", buffer, 1);
-		}
+		free(buffer);
 		fclose(fp);
-	} else
+	} else {
 		setenv("RC_DEFAULTLEVEL", RC_LEVEL_DEFAULT, 1);
+	}
 
-	free(buffer);
 	if (sys)
 		setenv("RC_SYS", sys, 1);
 
-#ifdef PREFIX
-	setenv("RC_PREFIX", RC_PREFIX, 1);
-#endif
-
 	/* Some scripts may need to take a different code path if
 	   Linux/FreeBSD, etc
 	   To save on calling uname, we store it in an environment variable */
@@ -248,7 +251,7 @@ svc_lock(const char *applet, bool ignore_lock_failure)
 	char *file = NULL;
 	int fd;
 
-	xasprintf(&file, RC_SVCDIR "/exclusive/%s", applet);
+	xasprintf(&file, "%s/exclusive/%s", rc_svcdir(), applet);
 	fd = open(file, O_WRONLY | O_CREAT | O_NONBLOCK, 0664);
 	free(file);
 	if (fd == -1)
@@ -274,7 +277,7 @@ svc_unlock(const char *applet, int fd)
 {
 	char *file = NULL;
 
-	xasprintf(&file, RC_SVCDIR "/exclusive/%s", applet);
+	xasprintf(&file, "%s/exclusive/%s", rc_svcdir(), applet);
 	close(fd);
 	unlink(file);
 	free(file);
@@ -382,18 +385,22 @@ RC_DEPTREE * _rc_deptree_load(int force, int *regen)
 	int merrno;
 	time_t t;
 	char file[PATH_MAX];
+	const char *svcdir = rc_svcdir();
 	struct stat st;
 	struct utimbuf ut;
 	FILE *fp;
 
 	t = 0;
 	if (rc_deptree_update_needed(&t, file) || force != 0) {
+		char *deptree_cache, *deptree_skewed;
+		xasprintf(&deptree_cache, "%s/deptree", svcdir);
+
 		/* Test if we have permission to update the deptree */
-		fd = open(RC_DEPTREE_CACHE, O_WRONLY);
+		fd = open(deptree_cache, O_WRONLY);
 		merrno = errno;
 		errno = serrno;
 		if (fd == -1 && merrno == EACCES)
-			return rc_deptree_load();
+			goto out;
 		close(fd);
 
 		if (regen)
@@ -403,29 +410,33 @@ RC_DEPTREE * _rc_deptree_load(int force, int *regen)
 		eend (retval, "Failed to update the dependency tree");
 
 		if (retval == 0) {
-			if (stat(RC_DEPTREE_CACHE, &st) != 0) {
-				eerror("stat(%s): %s", RC_DEPTREE_CACHE, strerror(errno));
+			if (stat(deptree_cache, &st) != 0) {
+				eerror("stat(%s): %s", deptree_cache, strerror(errno));
+				free(deptree_cache);
 				return NULL;
 			}
+			xasprintf(&deptree_skewed, "%s/clock-skewed", svcdir);
 			if (st.st_mtime < t) {
-				eerror("Clock skew detected with `%s'", file);
-				eerrorn("Adjusting mtime of `" RC_DEPTREE_CACHE
-				    "' to %s", ctime(&t));
-				fp = fopen(RC_DEPTREE_SKEWED, "w");
+				eerror("Clock skew detected with '%s'", file);
+				eerrorn("Adjusting mtime of '%s' to %s", deptree_cache, ctime(&t));
+				fp = fopen(deptree_skewed, "w");
 				if (fp != NULL) {
 					fprintf(fp, "%s\n", file);
 					fclose(fp);
 				}
 				ut.actime = t;
 				ut.modtime = t;
-				utime(RC_DEPTREE_CACHE, &ut);
+				utime(deptree_cache, &ut);
 			} else {
-				if (exists(RC_DEPTREE_SKEWED))
-					unlink(RC_DEPTREE_SKEWED);
+				if (exists(deptree_skewed))
+					unlink(deptree_skewed);
 			}
+			free(deptree_skewed);
 		}
 		if (force == -1 && regen != NULL)
 			*regen = retval;
+out:
+		free(deptree_cache);
 	}
 	return rc_deptree_load();
 }
