@@ -33,16 +33,7 @@
 #define RC_LEVEL_BOOT           "boot"
 #define RC_LEVEL_DEFAULT        "default"
 
-#define RC_DEPTREE_CACHE        RC_SVCDIR "/deptree"
-#define RC_DEPTREE_SKEWED	RC_SVCDIR "/clock-skewed"
 #define RC_KRUNLEVEL            RC_SVCDIR "/krunlevel"
-#define RC_STARTING             RC_SVCDIR "/rc.starting"
-#define RC_STOPPING             RC_SVCDIR "/rc.stopping"
-
-#define RC_SVCDIR_STARTING      RC_SVCDIR "/starting"
-#define RC_SVCDIR_INACTIVE      RC_SVCDIR "/inactive"
-#define RC_SVCDIR_STARTED       RC_SVCDIR "/started"
-#define RC_SVCDIR_COLDPLUGGED	RC_SVCDIR "/coldplugged"
 
 char *rc_conf_value(const char *var);
 bool rc_conf_yesno(const char *var);
