@@ -18,6 +18,11 @@
 #ifndef __LIBRC_PLUGIN_H__
 #define __LIBRC_PLUGIN_H__
 
+#include <stdbool.h>
+#include <sys/types.h>
+
+#include "rc.h"
+
 /* A simple flag to say if we're in a plugin process or not.
  * Mainly used in atexit code. */
 extern bool rc_in_plugin;
@@ -27,15 +32,4 @@ void rc_plugin_load(void);
 void rc_plugin_unload(void);
 void rc_plugin_run(RC_HOOK, const char *value);
 
-/* dlfunc defines needed to avoid ISO errors. FreeBSD has this right :) */
-#if !defined(__FreeBSD__) && !defined(__DragonFly__)
-struct __dlfunc_arg {
-	int	__dlfunc_dummy;
-};
-
-typedef	void (*dlfunc_t)(struct __dlfunc_arg);
-
-dlfunc_t dlfunc (void * __restrict handle, const char * __restrict symbol);
-#endif
-
 #endif
