@@ -26,47 +26,46 @@
 # define _GNU_SOURCE
 #endif
 
-#include <sys/types.h>
-#include <sys/ioctl.h>
-#include <sys/resource.h>
-#include <sys/stat.h>
-#include <termios.h>
-#include <sys/time.h>
-#include <sys/wait.h>
-
-#ifdef __linux__
-#include <sys/syscall.h> /* For io priority */
-#include <sys/prctl.h> /* For prctl */
-#endif
-
-#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
 #include <limits.h>
 #include <grp.h>
+#include <pthread.h>
 #include <pwd.h>
+#include <sched.h>
 #include <signal.h>
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
+#include <sys/ioctl.h>
+#include <sys/resource.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#ifdef __linux__
+# include <sys/syscall.h> /* For io priority */
+# include <sys/prctl.h> /* For prctl */
+#endif
+#include <termios.h>
 #include <time.h>
 #include <unistd.h>
 
 #ifdef HAVE_PAM
-#include <security/pam_appl.h>
+# include <security/pam_appl.h>
 
 /* We are not supporting authentication conversations */
 static struct pam_conv conv = { NULL, NULL};
 #endif
 
-#ifdef HAVE_CAP
-#include <sys/capability.h>
+#ifdef __linux__
+# include <sys/capability.h>
 #endif
 
-#include <sched.h>
-
 #include "einfo.h"
 #include "queue.h"
 #include "rc.h"
@@ -182,9 +181,9 @@ extern char **environ;
 # define SYS_ioprio_set __NR_ioprio_set
 #endif
 #if !defined(__DragonFly__)
-static inline int ioprio_set(int which _unused,
-			     int who _unused,
-			     int ioprio _unused)
+static inline int ioprio_set(int which RC_UNUSED,
+			     int who RC_UNUSED,
+			     int ioprio RC_UNUSED)
 {
 #ifdef SYS_ioprio_set
 	return syscall(SYS_ioprio_set, which, who, ioprio);
@@ -207,20 +206,20 @@ handle_signal(int sig)
 {
 	int status;
 	int serrno = errno;
-	char *signame = NULL;
+	const char *signame = NULL;
 
 	switch (sig) {
 	case SIGINT:
 		if (!signame)
-			xasprintf(&signame, "SIGINT");
+			signame = "SIGINT";
 		/* FALLTHROUGH */
 	case SIGTERM:
 		if (!signame)
-			xasprintf(&signame, "SIGTERM");
+			signame = "SIGTERM";
 		/* FALLTHROUGH */
 	case SIGQUIT:
 		if (!signame)
-			xasprintf(&signame, "SIGQUIT");
+			signame = "SIGQUIT";
 		eerrorx("%s: caught %s, aborting", applet, signame);
 		/* NOTREACHED */
 
@@ -239,9 +238,6 @@ handle_signal(int sig)
 		eerror("%s: caught unknown signal %d", applet, sig);
 	}
 
-	/* free signame */
-	free(signame);
-
 	/* Restore errno */
 	errno = serrno;
 }
@@ -347,13 +343,16 @@ int main(int argc, char **argv)
 	unsigned int start_wait = 0;
 	const char *scheduler = NULL;
 	int sched_prio = -1;
-#ifdef HAVE_CAP
+#ifdef __linux__
 	cap_iab_t cap_iab = NULL;
 	unsigned secbits = 0;
 #endif
 #ifdef PR_SET_NO_NEW_PRIVS
 	bool no_new_privs = false;
 #endif
+	int pipefd[2];
+	char readbuf[1];
+	ssize_t ss;
 
 	applet = basename_c(argv[0]);
 	atexit(cleanup);
@@ -362,6 +361,8 @@ int main(int argc, char **argv)
 	signal_setup(SIGQUIT, handle_signal);
 	signal_setup(SIGTERM, handle_signal);
 
+	openlog(applet, LOG_PID, LOG_DAEMON);
+
 	if ((tmp = getenv("SSD_NICELEVEL")))
 		if (sscanf(tmp, "%d", &nicelevel) != 1)
 			eerror("%s: invalid nice level `%s' (SSD_NICELEVEL)",
@@ -401,7 +402,7 @@ int main(int argc, char **argv)
 		    (int *) 0)) != -1)
 		switch (opt) {
 		case LONGOPT_CAPABILITIES:
-#ifdef HAVE_CAP
+#ifdef __linux__
 			cap_iab = cap_iab_from_text(optarg);
 			if (cap_iab == NULL)
 				eerrorx("Could not parse iab: %s", strerror(errno));
@@ -411,7 +412,7 @@ int main(int argc, char **argv)
 			break;
 
 		case LONGOPT_SECBITS:
-#ifdef HAVE_CAP
+#ifdef __linux__
 			if (*optarg == '\0')
 				eerrorx("Secbits are empty");
 
@@ -744,7 +745,7 @@ int main(int argc, char **argv)
 		fp = fopen(exec_file, "r");
 		if (fp) {
 			line = NULL;
-			if (getline(&line, &size, fp) == -1)
+			if (xgetline(&line, &size, fp) == -1)
 				eerrorx("%s: %s", applet, strerror(errno));
 			p = line;
 			fclose(fp);
@@ -753,10 +754,6 @@ int main(int argc, char **argv)
 				/* Strip leading spaces */
 				while (*p == ' ' || *p == '\t')
 					p++;
-				/* Remove the trailing newline */
-				len = strlen(p) - 1;
-				if (p[len] == '\n')
-					p[len] = '\0';
 				token = strsep(&p, " ");
 				free(exec_file);
 				xasprintf(&exec_file, "%s", token);
@@ -864,12 +861,17 @@ int main(int argc, char **argv)
 	if (background)
 		signal_setup(SIGCHLD, handle_signal);
 
+	/* Use a pipe to sync the parent/child processes. */
+	if (pipe2(pipefd, O_CLOEXEC) == -1)
+		eerrorx("%s: pipe2: %s", applet, strerror(errno));
+
 	if ((pid = fork()) == -1)
 		eerrorx("%s: fork: %s", applet, strerror(errno));
 
 	/* Child process - lets go! */
 	if (pid == 0) {
 		pid_t mypid = getpid();
+		close(pipefd[0]); /* Close the read end of the pipe. */
 		umask(numask);
 
 #ifdef TIOCNOTTY
@@ -878,11 +880,23 @@ int main(int argc, char **argv)
 
 		devnull_fd = open("/dev/null", O_RDWR);
 
+		/* Must call setsid() before setting autogroup nicelevel
+		 * but after opening tty_fd. */
+		setsid();
+
 		if (nicelevel != INT_MIN) {
 			if (setpriority(PRIO_PROCESS, mypid, nicelevel) == -1)
 				eerrorx("%s: setpriority %d: %s",
 				    applet, nicelevel,
 				    strerror(errno));
+			/* Open in "r+" mode to avoid creating if non-existent. */
+			fp = fopen("/proc/self/autogroup", "r+");
+			if (fp) {
+				fprintf(fp, "%d\n", nicelevel);
+				fclose(fp);
+			} else if (errno != ENOENT)
+				eerrorx("%s: autogroup nice %d: %s", applet,
+				    nicelevel, strerror(errno));
 		}
 
 		if (ionicec != -1 &&
@@ -937,7 +951,7 @@ int main(int argc, char **argv)
 		if (changeuser && initgroups(changeuser, gid))
 			eerrorx("%s: initgroups (%s, %d)",
 			    applet, changeuser, gid);
-#ifdef HAVE_CAP
+#ifdef __linux__
 		if (uid && cap_setuid(uid))
 #else
 		if (uid && setuid(uid))
@@ -948,7 +962,7 @@ int main(int argc, char **argv)
 		/* Close any fd's to the passwd database */
 		endpwent();
 
-#ifdef HAVE_CAP
+#ifdef __linux__
 		if (cap_iab != NULL) {
 			i = cap_iab_set_proc(cap_iab);
 
@@ -1084,8 +1098,7 @@ int main(int argc, char **argv)
 				|| rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stderr_fd, STDERR_FILENO);
 
-		for (i = getdtablesize() - 1; i >= 3; --i)
-			close(i);
+		cloexec_fds_from(3);
 
 		if (scheduler != NULL) {
 			int scheduler_index;
@@ -1110,7 +1123,7 @@ int main(int argc, char **argv)
 			if (sched_prio == -1)
 				sched.sched_priority = sched_get_priority_min(scheduler_index);
 
-			if (sched_setscheduler(mypid, scheduler_index, &sched))
+			if (pthread_setschedparam(pthread_self(), scheduler_index, &sched))
 				eerrorx("Failed to set scheduler: %s", strerror(errno));
 		} else if (sched_prio != -1) {
 			const struct sched_param sched =  {.sched_priority = sched_prio};
@@ -1118,7 +1131,6 @@ int main(int argc, char **argv)
 				eerrorx("Failed to set scheduler parameters: %s", strerror(errno));
 		}
 
-		setsid();
 		execvp(exec, argv);
 #ifdef HAVE_PAM
 		if (changeuser != NULL && pamr == PAM_SUCCESS)
@@ -1129,6 +1141,18 @@ int main(int argc, char **argv)
 	}
 
 	/* Parent process */
+
+	close(pipefd[1]); /* Close the write end of the pipe. */
+
+	/* The child never writes to the pipe, so this read will block until
+	 * the child calls exec or exits. */
+	while ((ss = read(pipefd[0], readbuf, 1)) == -1 && errno == EINTR);
+	if (ss == -1)
+		eerrorx("%s: failed to read from pipe: %s",
+			applet, strerror(errno));
+
+	close(pipefd[0]);
+
 	if (!background) {
 		/* As we're not backgrounding the process, wait for our pid
 		 * to return */
