@@ -22,20 +22,6 @@
 #define ONE_SECOND    1000000000
 #define ONE_MS           1000000
 
-#include <sys/types.h>
-#include <sys/ioctl.h>
-#include <sys/resource.h>
-#include <sys/stat.h>
-#include <termios.h>
-#include <sys/time.h>
-#include <sys/wait.h>
-
-#ifdef __linux__
-#include <sys/syscall.h> /* For io priority */
-#include <sys/prctl.h> /* For prctl */
-#endif
-
-#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -47,7 +33,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
+#include <strings.h>
+#ifdef __linux__
+# include <sys/syscall.h> /* For io priority */
+# include <sys/prctl.h> /* For prctl */
+#endif
 #include <syslog.h>
+#include <sys/ioctl.h>
+#include <sys/resource.h>
+#include <sys/stat.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -58,7 +54,7 @@
 static struct pam_conv conv = { NULL, NULL};
 #endif
 
-#ifdef HAVE_CAP
+#ifdef __linux__
 #include <sys/capability.h>
 #endif
 
@@ -66,6 +62,7 @@ static struct pam_conv conv = { NULL, NULL};
 #include "queue.h"
 #include "rc.h"
 #include "misc.h"
+#include "pipes.h"
 #include "plugin.h"
 #include "schedules.h"
 #include "_usage.h"
@@ -83,6 +80,8 @@ enum {
   LONGOPT_OOM_SCORE_ADJ,
   LONGOPT_NO_NEW_PRIVS,
   LONGOPT_SECBITS,
+  LONGOPT_STDERR_LOGGER,
+  LONGOPT_STDOUT_LOGGER,
 };
 
 const char *applet = NULL;
@@ -114,6 +113,8 @@ const struct option longopts[] = {
 	{ "user",         1, NULL, 'u'},
 	{ "stdout",       1, NULL, '1'},
 	{ "stderr",       1, NULL, '2'},
+	{ "stdout-logger",1, NULL, LONGOPT_STDOUT_LOGGER},
+	{ "stderr-logger",1, NULL, LONGOPT_STDERR_LOGGER},
 	{ "reexec",       0, NULL, '3'},
 	longopts_COMMON
 };
@@ -142,6 +143,8 @@ const char * const longopts_help[] = {
 	"Change the process user",
 	"Redirect stdout to file",
 	"Redirect stderr to file",
+	"Redirect stdout to process",
+	"Redirect stderr to process",
 	"reexec (used internally)",
 	longopts_help_COMMON
 };
@@ -164,6 +167,8 @@ static int stdout_fd;
 static int stderr_fd;
 static char *redirect_stderr = NULL;
 static char *redirect_stdout = NULL;
+static char *stderr_process = NULL;
+static char *stdout_process = NULL;
 #ifdef TIOCNOTTY
 static int tty_fd = -1;
 #endif
@@ -177,7 +182,7 @@ static int fifo_fd = 0;
 static char *pidfile = NULL;
 static char *svcname = NULL;
 static bool verbose = false;
-#ifdef HAVE_CAP
+#ifdef __linux__
 static cap_iab_t cap_iab = NULL;
 static unsigned secbits = 0;
 #endif
@@ -191,8 +196,8 @@ extern char **environ;
 # define SYS_ioprio_set __NR_ioprio_set
 #endif
 #if !defined(__DragonFly__)
-static inline int ioprio_set(int which _unused, int who _unused,
-			     int ioprio _unused)
+static inline int ioprio_set(int which RC_UNUSED, int who RC_UNUSED,
+			     int ioprio RC_UNUSED)
 {
 #ifdef SYS_ioprio_set
 	return syscall(SYS_ioprio_set, which, who, ioprio);
@@ -207,7 +212,7 @@ static void cleanup(void)
 	free(changeuser);
 }
 
-static void re_exec_supervisor(void)
+RC_NORETURN static void re_exec_supervisor(void)
 {
 	syslog(LOG_WARNING, "Re-executing for %s", svcname);
 	execlp("supervise-daemon", "supervise-daemon", svcname, "--reexec",
@@ -353,7 +358,7 @@ static pid_t exec_command(const char *cmd)
 	return pid;
 }
 
-static void child_process(char *exec, char **argv)
+RC_NORETURN static void child_process(char *exec, char **argv)
 {
 	RC_STRINGLIST *env_list;
 	RC_STRING *env;
@@ -391,6 +396,14 @@ static void child_process(char *exec, char **argv)
 		if (setpriority(PRIO_PROCESS, getpid(), nicelevel) == -1)
 			eerrorx("%s: setpriority %d: %s", applet, nicelevel,
 					strerror(errno));
+		/* Open in "r+" mode to avoid creating if non-existent. */
+		fp = fopen("/proc/self/autogroup", "r+");
+		if (fp) {
+			fprintf(fp, "%d\n", nicelevel);
+			fclose(fp);
+		} else if (errno != ENOENT)
+			eerrorx("%s: autogroup nice %d: %s", applet,
+			    nicelevel, strerror(errno));
 	}
 
 	if (ionicec != -1 && ioprio_set(1, getpid(), ionicec | ioniced) == -1)
@@ -430,7 +443,7 @@ static void child_process(char *exec, char **argv)
 		eerrorx("%s: unable to set groupid to %d", applet, gid);
 	if (changeuser && initgroups(changeuser, gid))
 		eerrorx("%s: initgroups (%s, %d)", applet, changeuser, gid);
-#ifdef HAVE_CAP
+#ifdef __linux__
 	if (uid && cap_setuid(uid))
 #else
 	if (uid && setuid(uid))
@@ -440,7 +453,7 @@ static void child_process(char *exec, char **argv)
 	/* Close any fd's to the passwd database */
 	endpwent();
 
-#ifdef HAVE_CAP
+#ifdef __linux__
 	if (cap_iab != NULL) {
 		i = cap_iab_set_proc(cap_iab);
 
@@ -545,6 +558,12 @@ static void child_process(char *exec, char **argv)
 			eerrorx("%s: unable to open the logfile"
 				    " for stdout `%s': %s",
 				    applet, redirect_stdout, strerror(errno));
+	} else if (stdout_process) {
+		stdout_fd = rc_pipe_command(stdout_process);
+		if (stdout_fd == -1)
+			eerrorx("%s: unable to open the logging process"
+			    " for stdout `%s': %s",
+			    applet, stdout_process, strerror(errno));
 	}
 	if (redirect_stderr) {
 		if ((stderr_fd = open(redirect_stderr,
@@ -553,16 +572,22 @@ static void child_process(char *exec, char **argv)
 			eerrorx("%s: unable to open the logfile"
 			    " for stderr `%s': %s",
 			    applet, redirect_stderr, strerror(errno));
+	} else if (stderr_process) {
+		stderr_fd = rc_pipe_command(stderr_process);
+		if (stderr_fd == -1)
+			eerrorx("%s: unable to open the logging process"
+			    " for stderr `%s': %s",
+			    applet, stderr_process, strerror(errno));
 	}
 
 	dup2(stdin_fd, STDIN_FILENO);
-	if (redirect_stdout || rc_yesno(getenv("EINFO_QUIET")))
+	if (redirect_stdout || stdout_process || rc_yesno(getenv("EINFO_QUIET")))
 		dup2(stdout_fd, STDOUT_FILENO);
-	if (redirect_stderr || rc_yesno(getenv("EINFO_QUIET")))
+	if (redirect_stderr || stderr_process || rc_yesno(getenv("EINFO_QUIET")))
 		dup2(stderr_fd, STDERR_FILENO);
 
-	for (i = getdtablesize() - 1; i >= 3; --i)
-		fcntl(i, F_SETFD, FD_CLOEXEC);
+	cloexec_fds_from(3);
+
 	cmdline = make_cmdline(argv);
 	syslog(LOG_INFO, "Child command line: %s", cmdline);
 	free(cmdline);
@@ -575,7 +600,7 @@ static void child_process(char *exec, char **argv)
 	eerrorx("%s: failed to exec `%s': %s", applet, exec,strerror(errno));
 }
 
-static void supervisor(char *exec, char **argv)
+RC_NORETURN static void supervisor(char *exec, char **argv)
 {
 	FILE *fp;
 	char buf[2048];
@@ -858,7 +883,7 @@ int main(int argc, char **argv)
 			break;
 
 		case LONGOPT_CAPABILITIES:
-#ifdef HAVE_CAP
+#ifdef __linux__
 			cap_iab = cap_iab_from_text(optarg);
 			if (cap_iab == NULL)
 				eerrorx("Could not parse iab: %s", strerror(errno));
@@ -868,7 +893,7 @@ int main(int argc, char **argv)
 			break;
 
         case LONGOPT_SECBITS:
-#ifdef HAVE_CAP
+#ifdef __linux__
 			if (*optarg == '\0')
 				eerrorx("Secbits are empty");
 
@@ -1035,6 +1060,14 @@ int main(int argc, char **argv)
 			reexec = true;
 			break;
 
+		case LONGOPT_STDOUT_LOGGER:   /* --stdout-logger "command to run for stdout logging" */
+			stdout_process = optarg;
+			break;
+
+		case LONGOPT_STDERR_LOGGER:  /* --stderr-logger "command to run for stderr logging" */
+			stderr_process = optarg;
+			break;
+
 		case_RC_COMMON_GETOPT
 		}
 
@@ -1053,7 +1086,7 @@ int main(int argc, char **argv)
 	umask(numask);
 	if (!pidfile)
 		xasprintf(&pidfile, "/var/run/supervise-%s.pid", svcname);
-	xasprintf(&fifopath, "%s/supervise-%s.ctl", RC_SVCDIR, svcname);
+	xasprintf(&fifopath, "%s/supervise-%s.ctl", rc_svcdir(), svcname);
 	if (mkfifo(fifopath, 0600) == -1 && errno != EEXIST)
 		eerrorx("%s: unable to create control fifo: %s",
 				applet, strerror(errno));
