@@ -12,24 +12,14 @@
 
 #define SYSLOG_NAMES
 
-#include <sys/types.h>
-#include <sys/time.h>
-
-#include <errno.h>
-#include <ctype.h>
-#include <inttypes.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <signal.h>
 #include <string.h>
-#include <syslog.h>
-#include <time.h>
-#include <unistd.h>
 
 #include "einfo.h"
 #include "rc.h"
-#include "misc.h"
+#include "helpers.h"
 
 const char *applet = NULL;
 
